@@ -304,7 +304,7 @@ pub fn metadata(contract_no: usize, code: &[u8], ns: &ast::Namespace) -> Value {
     let mut builder = Contract::builder();
 
     // Add our name and tags
-    builder.name(ns.contracts[contract_no].name.to_string());
+    builder.name(&ns.contracts[contract_no].name);
 
     let mut description = tags(contract_no, "title", ns);
 
@@ -462,11 +462,7 @@ fn gen_abi(contract_no: usize, ns: &ast::Namespace) -> Abi {
                             .returns
                             .iter()
                             .map(|f| StructField {
-                                name: if f.name.is_empty() {
-                                    None
-                                } else {
-                                    Some(f.name.to_string())
-                                },
+                                name: f.id.as_ref().map(|id| id.name.to_owned()),
                                 ty: ty_to_abi(&f.ty, ns, &mut abi).ty,
                             })
                             .collect();
@@ -531,7 +527,7 @@ fn ty_to_abi(ty: &ast::Type, ns: &ast::Namespace, registry: &mut Abi) -> ParamTy
                 display_name: vec![],
             }
         }
-        ast::Type::Mapping(_, _) => unreachable!(),
+        ast::Type::Mapping(..) => unreachable!(),
         ast::Type::Array(ty, dims) => {
             let mut param_ty = ty_to_abi(ty, ns, registry);
 
@@ -553,11 +549,14 @@ fn ty_to_abi(ty: &ast::Type, ns: &ast::Namespace, registry: &mut Abi) -> ParamTy
         }
         ast::Type::StorageRef(_, ty) => ty_to_abi(ty, ns, registry),
         ast::Type::Ref(ty) => ty_to_abi(ty, ns, registry),
+        ast::Type::UserType(no) => ty_to_abi(&ns.user_types[*no].ty, ns, registry),
         ast::Type::Bool | ast::Type::Uint(_) | ast::Type::Int(_) => {
             let scalety = match ty {
                 ast::Type::Bool => "bool".into(),
-                ast::Type::Uint(n) => format!("u{}", n),
-                ast::Type::Int(n) => format!("i{}", n),
+                // Substrate doesn't like primitive types which aren't a power of 2
+                // The abi encoder/decoder fixes this automatically
+                ast::Type::Uint(n) => format!("u{}", n.next_power_of_two()),
+                ast::Type::Int(n) => format!("i{}", n.next_power_of_two()),
                 _ => unreachable!(),
             };
 
@@ -590,7 +589,7 @@ fn ty_to_abi(ty: &ast::Type, ns: &ast::Namespace, registry: &mut Abi) -> ParamTy
                 .fields
                 .iter()
                 .map(|f| StructField {
-                    name: Some(f.name.to_string()),
+                    name: Some(f.name_as_str().to_owned()),
                     ty: ty_to_abi(&f.ty, ns, registry).ty,
                 })
                 .collect();
@@ -641,7 +640,7 @@ fn ty_to_abi(ty: &ast::Type, ns: &ast::Namespace, registry: &mut Abi) -> ParamTy
 
 fn parameter_to_abi(param: &ast::Parameter, ns: &ast::Namespace, registry: &mut Abi) -> Param {
     Param {
-        name: param.name.to_string(),
+        name: param.name_as_str().to_owned(),
         ty: ty_to_abi(&param.ty, ns, registry),
     }
 }
