@@ -1,19 +1,17 @@
+use clap::ArgMatches;
 use serde_json::Value;
-use std::cmp::Ordering;
-use std::collections::HashMap;
-use std::path::PathBuf;
+use solang::{
+    codegen,
+    codegen::codegen,
+    file_resolver::FileResolver,
+    parse_and_resolve,
+    parser::pt,
+    sema::{ast, builtin::get_prototype, symtable, tags::render},
+    Target,
+};
+use std::{cmp::Ordering, collections::HashMap, ffi::OsString, path::PathBuf};
 use tokio::sync::Mutex;
-use tower_lsp::jsonrpc::Result;
-use tower_lsp::lsp_types::*;
-use tower_lsp::{Client, LanguageServer};
-use tower_lsp::{LspService, Server};
-
-use solang::codegen::codegen;
-use solang::file_resolver::FileResolver;
-use solang::parse_and_resolve;
-use solang::parser::pt;
-use solang::sema::{ast, builtin::get_prototype, symtable, tags::render};
-use solang::Target;
+use tower_lsp::{jsonrpc::Result, lsp_types::*, Client, LanguageServer, LspService, Server};
 
 pub struct Hovers {
     file: ast::File,
@@ -23,24 +21,23 @@ pub struct Hovers {
 pub struct SolangServer {
     client: Client,
     target: Target,
+    matches: ArgMatches,
     files: Mutex<HashMap<PathBuf, Hovers>>,
 }
 
 #[tokio::main(flavor = "current_thread")]
-pub async fn start_server(target: Target) {
+pub async fn start_server(target: Target, matches: ArgMatches) -> ! {
     let stdin = tokio::io::stdin();
     let stdout = tokio::io::stdout();
 
-    let (service, messages) = LspService::new(|client| SolangServer {
+    let (service, socket) = LspService::new(|client| SolangServer {
         client,
         target,
         files: Mutex::new(HashMap::new()),
+        matches,
     });
 
-    Server::new(stdin, stdout)
-        .interleave(messages)
-        .serve(service)
-        .await;
+    Server::new(stdin, stdout, socket).serve(service).await;
 
     std::process::exit(1);
 }
@@ -55,66 +52,95 @@ impl SolangServer {
 
             let _ = resolver.add_import_path(PathBuf::from(dir));
 
+            let mut diags = Vec::new();
+
+            if let Some(paths) = self.matches.values_of_os("IMPORTPATH") {
+                for path in paths {
+                    if let Err(e) = resolver.add_import_path(PathBuf::from(path)) {
+                        diags.push(Diagnostic {
+                            message: format!("import path '{}': {}", path.to_string_lossy(), e),
+                            severity: Some(DiagnosticSeverity::ERROR),
+                            ..Default::default()
+                        });
+                    }
+                }
+            }
+
+            if let Some(maps) = self.matches.values_of("IMPORTMAP") {
+                for p in maps {
+                    if let Some((map, path)) = p.split_once('=') {
+                        if let Err(e) =
+                            resolver.add_import_map(OsString::from(map), PathBuf::from(path))
+                        {
+                            diags.push(Diagnostic {
+                                message: format!("error: import path '{}': {}", path, e),
+                                severity: Some(DiagnosticSeverity::ERROR),
+                                ..Default::default()
+                            });
+                        }
+                    } else {
+                        diags.push(Diagnostic {
+                            message: format!("error: import map '{}': contains no '='", p),
+                            severity: Some(DiagnosticSeverity::ERROR),
+                            ..Default::default()
+                        });
+                    }
+                }
+            }
+
             let os_str = path.file_name().unwrap();
 
-            let mut ns = parse_and_resolve(os_str.to_str().unwrap(), &mut resolver, self.target);
+            let mut ns = parse_and_resolve(os_str, &mut resolver, self.target);
 
             // codegen all the contracts; some additional errors/warnings will be detected here
             codegen(&mut ns, &Default::default());
 
-            let diags = ns
-                .diagnostics
-                .iter()
-                .filter_map(|diag| {
-                    let pos = diag.pos.unwrap();
+            diags.extend(ns.diagnostics.iter().filter_map(|diag| {
+                if diag.pos.file_no() != ns.top_file_no() {
+                    // The first file is the one we wanted to parse; others are imported
+                    return None;
+                }
 
-                    if pos.0 != 0 {
-                        // The first file is the one we wanted to parse; others are imported
+                let severity = match diag.level {
+                    ast::Level::Info => Some(DiagnosticSeverity::INFORMATION),
+                    ast::Level::Warning => Some(DiagnosticSeverity::WARNING),
+                    ast::Level::Error => Some(DiagnosticSeverity::ERROR),
+                    ast::Level::Debug => {
                         return None;
                     }
+                };
 
-                    let related_information = if diag.notes.is_empty() {
-                        None
-                    } else {
-                        Some(
-                            diag.notes
-                                .iter()
-                                .map(|note| DiagnosticRelatedInformation {
-                                    message: note.message.to_string(),
-                                    location: Location {
-                                        uri: Url::from_file_path(&ns.files[note.pos.0].path)
-                                            .unwrap(),
-                                        range: SolangServer::loc_to_range(&note.pos, &ns.files[0]),
-                                    },
-                                })
-                                .collect(),
-                        )
-                    };
+                let related_information = if diag.notes.is_empty() {
+                    None
+                } else {
+                    Some(
+                        diag.notes
+                            .iter()
+                            .map(|note| DiagnosticRelatedInformation {
+                                message: note.message.to_string(),
+                                location: Location {
+                                    uri: Url::from_file_path(&ns.files[note.pos.file_no()].path)
+                                        .unwrap(),
+                                    range: SolangServer::loc_to_range(
+                                        &note.pos,
+                                        &ns.files[ns.top_file_no()],
+                                    ),
+                                },
+                            })
+                            .collect(),
+                    )
+                };
 
-                    let sev = match diag.level {
-                        ast::Level::Info => DiagnosticSeverity::Information,
-                        ast::Level::Warning => DiagnosticSeverity::Warning,
-                        ast::Level::Error => DiagnosticSeverity::Error,
-                        ast::Level::Debug => {
-                            return None;
-                        }
-                    };
+                let range = SolangServer::loc_to_range(&diag.pos, &ns.files[ns.top_file_no()]);
 
-                    let range = SolangServer::loc_to_range(&pos, &ns.files[0]);
-
-                    Some(Diagnostic {
-                        range,
-                        message: diag.message.to_string(),
-                        severity: Some(sev),
-                        source: None,
-                        code: None,
-                        code_description: None,
-                        related_information,
-                        tags: None,
-                        data: None,
-                    })
+                Some(Diagnostic {
+                    range,
+                    message: diag.message.to_string(),
+                    severity,
+                    related_information,
+                    ..Default::default()
                 })
-                .collect();
+            }));
 
             let res = self.client.publish_diagnostics(uri, diags, None);
 
@@ -128,7 +154,7 @@ impl SolangServer {
             self.files.lock().await.insert(
                 path,
                 Hovers {
-                    file: ns.files[0].clone(),
+                    file: ns.files[ns.top_file_no()].clone(),
                     lookup,
                 },
             );
@@ -139,9 +165,9 @@ impl SolangServer {
 
     /// Calculate the line and column from the Loc offset received from the parser
     fn loc_to_range(loc: &pt::Loc, file: &ast::File) -> Range {
-        let (line, column) = file.offset_to_line_column(loc.1);
+        let (line, column) = file.offset_to_line_column(loc.start());
         let start = Position::new(line as u32, column as u32);
-        let (line, column) = file.offset_to_line_column(loc.2);
+        let (line, column) = file.offset_to_line_column(loc.end());
         let end = Position::new(line as u32, column as u32);
 
         Range::new(start, end)
@@ -152,11 +178,11 @@ impl SolangServer {
         let prot = get_prototype(*bltn);
 
         if let Some(protval) = prot {
-            for ret in protval.ret {
+            for ret in &protval.ret {
                 msg = format!("{} {}", msg, SolangServer::expanded_ty(ret, ns));
             }
             msg = format!("{} {} (", msg, protval.name);
-            for arg in protval.args {
+            for arg in &protval.args {
                 msg = format!("{}{}", msg, SolangServer::expanded_ty(arg, ns));
             }
             msg = format!("{}): {}", msg, protval.doc);
@@ -184,18 +210,18 @@ impl SolangServer {
                     SolangServer::construct_expr(exp, lookup_tbl, symtab, fnc_map, ns);
                 }
                 let mut msg = SolangServer::expanded_ty(&_param.ty, ns);
-                msg = format!("{} {}", msg, _param.name);
+                msg = format!("{} {}", msg, _param.name_as_str());
                 if let Some(expr) = ns.var_constants.get(loc) {
                     match expr {
-                        ast::Expression::BytesLiteral(_, ast::Type::Bytes(_), bs)
-                        | ast::Expression::BytesLiteral(_, ast::Type::DynamicBytes, bs) => {
+                        codegen::Expression::BytesLiteral(_, ast::Type::Bytes(_), bs)
+                        | codegen::Expression::BytesLiteral(_, ast::Type::DynamicBytes, bs) => {
                             msg.push_str(&format!(" = hex\"{}\"", hex::encode(&bs)));
                         }
-                        ast::Expression::BytesLiteral(_, ast::Type::String, bs) => {
+                        codegen::Expression::BytesLiteral(_, ast::Type::String, bs) => {
                             msg.push_str(&format!(" = \"{}\"", String::from_utf8_lossy(bs)));
                         }
-                        ast::Expression::NumberLiteral(_, ast::Type::Uint(_), n)
-                        | ast::Expression::NumberLiteral(_, ast::Type::Int(_), n) => {
+                        codegen::Expression::NumberLiteral(_, ast::Type::Uint(_), n)
+                        | codegen::Expression::NumberLiteral(_, ast::Type::Int(_), n) => {
                             msg.push_str(&format!(" = {}", n));
                         }
                         _ => (),
@@ -208,7 +234,7 @@ impl SolangServer {
                     }
                 }
 
-                lookup_tbl.push((_param.loc.1, _param.loc.2, msg));
+                lookup_tbl.push((_param.loc.start(), _param.loc.end(), msg));
             }
             ast::Statement::If(_locs, _, expr, stat1, stat2) => {
                 SolangServer::construct_expr(expr, lookup_tbl, symtab, fnc_map, ns);
@@ -271,8 +297,8 @@ impl SolangServer {
             }
             ast::Statement::Continue(_locs) => {}
             ast::Statement::Break(_) => {}
-            ast::Statement::Return(_locs, None) => {}
-            ast::Statement::Return(_locs, Some(expr)) => {
+            ast::Statement::Return(_, None) => {}
+            ast::Statement::Return(_, Some(expr)) => {
                 SolangServer::construct_expr(expr, lookup_tbl, symtab, fnc_map, ns);
             }
             ast::Statement::Emit {
@@ -293,7 +319,7 @@ impl SolangServer {
                         "\t{}{}{}{}\n",
                         field.ty.to_string(ns),
                         if field.indexed { " indexed " } else { " " },
-                        field.name,
+                        field.name_as_str(),
                         if iter.peek().is_some() { "," } else { "" }
                     ));
                 }
@@ -303,37 +329,30 @@ impl SolangServer {
                     if event.anonymous { " anonymous" } else { "" }
                 ));
 
-                lookup_tbl.push((event_loc.1, event_loc.2, msg));
+                lookup_tbl.push((event_loc.start(), event_loc.end(), msg));
 
                 for arg in args {
                     SolangServer::construct_expr(arg, lookup_tbl, symtab, fnc_map, ns);
                 }
             }
-            ast::Statement::TryCatch {
-                loc: _,
-                reachable: _,
-                expr,
-                returns: _,
-                ok_stmt,
-                error,
-                catch_param: _,
-                catch_param_pos: _,
-                catch_stmt,
-            } => {
-                SolangServer::construct_expr(expr, lookup_tbl, symtab, fnc_map, ns);
-                for vecstmt in catch_stmt {
+            ast::Statement::TryCatch(_, _, try_stmt) => {
+                SolangServer::construct_expr(&try_stmt.expr, lookup_tbl, symtab, fnc_map, ns);
+                for vecstmt in &try_stmt.catch_stmt {
                     SolangServer::construct_stmt(vecstmt, lookup_tbl, symtab, fnc_map, ns);
                 }
-                for vecstmt in ok_stmt {
+                for vecstmt in &try_stmt.ok_stmt {
                     SolangServer::construct_stmt(vecstmt, lookup_tbl, symtab, fnc_map, ns);
                 }
-                if let Some(okstmt) = error {
+                for okstmt in &try_stmt.errors {
                     for stmts in &okstmt.2 {
                         SolangServer::construct_stmt(stmts, lookup_tbl, symtab, fnc_map, ns);
                     }
                 }
             }
             ast::Statement::Underscore(_loc) => {}
+            ast::Statement::Assembly(..) => {
+                //unimplemented!("Assembly block not implemented in language server");
+            }
         }
     }
 
@@ -347,27 +366,22 @@ impl SolangServer {
         ns: &ast::Namespace,
     ) {
         match expr {
-            ast::Expression::FunctionArg(locs, typ, _sample_sz) => {
-                let msg = SolangServer::expanded_ty(typ, ns);
-                lookup_tbl.push((locs.1, locs.2, msg));
-            }
-
             // Variable types expression
             ast::Expression::BoolLiteral(locs, vl) => {
                 let msg = format!("(bool) {}", vl);
-                lookup_tbl.push((locs.1, locs.2, msg));
+                lookup_tbl.push((locs.start(), locs.end(), msg));
             }
             ast::Expression::BytesLiteral(locs, typ, _vec_lst) => {
                 let msg = format!("({})", typ.to_string(ns));
-                lookup_tbl.push((locs.1, locs.2, msg));
+                lookup_tbl.push((locs.start(), locs.end(), msg));
             }
             ast::Expression::CodeLiteral(locs, _val, _) => {
                 let msg = format!("({})", _val);
-                lookup_tbl.push((locs.1, locs.2, msg));
+                lookup_tbl.push((locs.start(), locs.end(), msg));
             }
             ast::Expression::NumberLiteral(locs, typ, _) => {
                 let msg = typ.to_string(ns);
-                lookup_tbl.push((locs.1, locs.2, msg));
+                lookup_tbl.push((locs.start(), locs.end(), msg));
             }
             ast::Expression::StructLiteral(_locs, _typ, expr) => {
                 for expp in expr {
@@ -388,8 +402,8 @@ impl SolangServer {
             // Arithmetic expression
             ast::Expression::Add(locs, ty, unchecked, expr1, expr2) => {
                 lookup_tbl.push((
-                    locs.1,
-                    locs.2,
+                    locs.start(),
+                    locs.end(),
                     format!(
                         "{} {}addition",
                         if *unchecked { "unchecked " } else { "" },
@@ -402,8 +416,8 @@ impl SolangServer {
             }
             ast::Expression::Subtract(locs, ty, unchecked, expr1, expr2) => {
                 lookup_tbl.push((
-                    locs.1,
-                    locs.2,
+                    locs.start(),
+                    locs.end(),
                     format!(
                         "{} {}subtraction",
                         if *unchecked { "unchecked " } else { "" },
@@ -416,8 +430,8 @@ impl SolangServer {
             }
             ast::Expression::Multiply(locs, ty, unchecked, expr1, expr2) => {
                 lookup_tbl.push((
-                    locs.1,
-                    locs.2,
+                    locs.start(),
+                    locs.end(),
                     format!(
                         "{} {}multiply",
                         if *unchecked { "unchecked " } else { "" },
@@ -429,21 +443,29 @@ impl SolangServer {
                 SolangServer::construct_expr(expr2, lookup_tbl, symtab, fnc_map, ns);
             }
             ast::Expression::Divide(locs, ty, expr1, expr2) => {
-                lookup_tbl.push((locs.1, locs.2, format!("{} divide", ty.to_string(ns))));
+                lookup_tbl.push((
+                    locs.start(),
+                    locs.end(),
+                    format!("{} divide", ty.to_string(ns)),
+                ));
 
                 SolangServer::construct_expr(expr1, lookup_tbl, symtab, fnc_map, ns);
                 SolangServer::construct_expr(expr2, lookup_tbl, symtab, fnc_map, ns);
             }
             ast::Expression::Modulo(locs, ty, expr1, expr2) => {
-                lookup_tbl.push((locs.1, locs.2, format!("{} modulo", ty.to_string(ns))));
+                lookup_tbl.push((
+                    locs.start(),
+                    locs.end(),
+                    format!("{} modulo", ty.to_string(ns)),
+                ));
 
                 SolangServer::construct_expr(expr1, lookup_tbl, symtab, fnc_map, ns);
                 SolangServer::construct_expr(expr2, lookup_tbl, symtab, fnc_map, ns);
             }
             ast::Expression::Power(locs, ty, unchecked, expr1, expr2) => {
                 lookup_tbl.push((
-                    locs.1,
-                    locs.2,
+                    locs.start(),
+                    locs.end(),
                     format!(
                         "{} {}power",
                         if *unchecked { "unchecked " } else { "" },
@@ -483,15 +505,15 @@ impl SolangServer {
 
                 if let Some(expr) = ns.var_constants.get(loc) {
                     match expr {
-                        ast::Expression::BytesLiteral(_, ast::Type::Bytes(_), bs)
-                        | ast::Expression::BytesLiteral(_, ast::Type::DynamicBytes, bs) => {
+                        codegen::Expression::BytesLiteral(_, ast::Type::Bytes(_), bs)
+                        | codegen::Expression::BytesLiteral(_, ast::Type::DynamicBytes, bs) => {
                             msg.push_str(&format!(" hex\"{}\"", hex::encode(&bs)));
                         }
-                        ast::Expression::BytesLiteral(_, ast::Type::String, bs) => {
+                        codegen::Expression::BytesLiteral(_, ast::Type::String, bs) => {
                             msg.push_str(&format!(" \"{}\"", String::from_utf8_lossy(bs)));
                         }
-                        ast::Expression::NumberLiteral(_, ast::Type::Uint(_), n)
-                        | ast::Expression::NumberLiteral(_, ast::Type::Int(_), n) => {
+                        codegen::Expression::NumberLiteral(_, ast::Type::Uint(_), n)
+                        | codegen::Expression::NumberLiteral(_, ast::Type::Int(_), n) => {
                             msg.push_str(&format!(" {}", n));
                         }
                         _ => (),
@@ -504,15 +526,15 @@ impl SolangServer {
                     }
                 }
 
-                lookup_tbl.push((loc.1, loc.2, msg));
+                lookup_tbl.push((loc.start(), loc.end(), msg));
             }
             ast::Expression::ConstantVariable(locs, typ, _val1, _val2) => {
                 let msg = format!("constant ({})", SolangServer::expanded_ty(typ, ns,));
-                lookup_tbl.push((locs.1, locs.2, msg));
+                lookup_tbl.push((locs.start(), locs.end(), msg));
             }
             ast::Expression::StorageVariable(locs, typ, _val1, _val2) => {
                 let msg = format!("({})", SolangServer::expanded_ty(typ, ns));
-                lookup_tbl.push((locs.1, locs.2, msg));
+                lookup_tbl.push((locs.start(), locs.end(), msg));
             }
 
             // Load expression
@@ -598,7 +620,7 @@ impl SolangServer {
                 SolangServer::construct_expr(expr3, lookup_tbl, symtab, fnc_map, ns);
             }
 
-            ast::Expression::Subscript(_locs, _typ, expr1, expr2) => {
+            ast::Expression::Subscript(_locs, _, _, expr1, expr2) => {
                 SolangServer::construct_expr(expr1, lookup_tbl, symtab, fnc_map, ns);
                 SolangServer::construct_expr(expr2, lookup_tbl, symtab, fnc_map, ns);
             }
@@ -611,24 +633,6 @@ impl SolangServer {
             ast::Expression::AllocDynamicArray(_locs, _typ, expr1, _valvec) => {
                 SolangServer::construct_expr(expr1, lookup_tbl, symtab, fnc_map, ns);
             }
-            ast::Expression::DynamicArrayLength(_locs, expr1) => {
-                SolangServer::construct_expr(expr1, lookup_tbl, symtab, fnc_map, ns);
-            }
-            ast::Expression::DynamicArraySubscript(_locs, _typ, expr1, expr2) => {
-                SolangServer::construct_expr(expr1, lookup_tbl, symtab, fnc_map, ns);
-                SolangServer::construct_expr(expr2, lookup_tbl, symtab, fnc_map, ns);
-            }
-            ast::Expression::DynamicArrayPush(_locs, expr1, _typ, expr2) => {
-                SolangServer::construct_expr(expr1, lookup_tbl, symtab, fnc_map, ns);
-                SolangServer::construct_expr(expr2, lookup_tbl, symtab, fnc_map, ns);
-            }
-            ast::Expression::DynamicArrayPop(_locs, expr1, _typ) => {
-                SolangServer::construct_expr(expr1, lookup_tbl, symtab, fnc_map, ns);
-            }
-            ast::Expression::StorageBytesSubscript(_locs, expr1, expr2) => {
-                SolangServer::construct_expr(expr1, lookup_tbl, symtab, fnc_map, ns);
-                SolangServer::construct_expr(expr2, lookup_tbl, symtab, fnc_map, ns);
-            }
             ast::Expression::StorageArrayLength { array, .. } => {
                 SolangServer::construct_expr(array, lookup_tbl, symtab, fnc_map, ns);
             }
@@ -673,10 +677,10 @@ impl SolangServer {
 
                     let mut param_msg = format!("{} \n\n {} {}(", msg_tg, fnc.ty, fnc.name);
 
-                    for parm in &fnc.params {
+                    for parm in &*fnc.params {
                         let msg = format!(
                             "{}:{}, \n\n",
-                            parm.name,
+                            parm.name_as_str(),
                             SolangServer::expanded_ty(&parm.ty, ns)
                         );
                         param_msg = format!("{} {}", param_msg, msg);
@@ -684,14 +688,17 @@ impl SolangServer {
 
                     param_msg = format!("{} ) returns (", param_msg);
 
-                    for ret in &fnc.returns {
-                        let msg =
-                            format!("{}:{}, ", ret.name, SolangServer::expanded_ty(&ret.ty, ns));
+                    for ret in &*fnc.returns {
+                        let msg = format!(
+                            "{}:{}, ",
+                            ret.name_as_str(),
+                            SolangServer::expanded_ty(&ret.ty, ns)
+                        );
                         param_msg = format!("{} {}", param_msg, msg);
                     }
 
                     param_msg = format!("{})", param_msg);
-                    lookup_tbl.push((loc.1, loc.2, param_msg));
+                    lookup_tbl.push((loc.start(), loc.end(), param_msg));
                 }
 
                 for arg in args {
@@ -702,8 +709,7 @@ impl SolangServer {
                 loc,
                 function,
                 args,
-                value,
-                gas,
+                call_args,
                 ..
             } => {
                 if let ast::Expression::ExternalFunction {
@@ -717,10 +723,10 @@ impl SolangServer {
                     let msg_tg = render(&fnc.tags[..]);
                     let mut param_msg = format!("{} \n\n {} {}(", msg_tg, fnc.ty, fnc.name);
 
-                    for parm in &fnc.params {
+                    for parm in &*fnc.params {
                         let msg = format!(
                             "{}:{}, \n\n",
-                            parm.name,
+                            parm.name_as_str(),
                             SolangServer::expanded_ty(&parm.ty, ns)
                         );
                         param_msg = format!("{} {}", param_msg, msg);
@@ -728,77 +734,71 @@ impl SolangServer {
 
                     param_msg = format!("{} ) \n\n returns (", param_msg);
 
-                    for ret in &fnc.returns {
-                        let msg =
-                            format!("{}:{}, ", ret.name, SolangServer::expanded_ty(&ret.ty, ns));
+                    for ret in &*fnc.returns {
+                        let msg = format!(
+                            "{}:{}, ",
+                            ret.name_as_str(),
+                            SolangServer::expanded_ty(&ret.ty, ns)
+                        );
                         param_msg = format!("{} {}", param_msg, msg);
                     }
 
                     param_msg = format!("{})", param_msg);
-                    lookup_tbl.push((loc.1, loc.2, param_msg));
+                    lookup_tbl.push((loc.start(), loc.end(), param_msg));
 
                     SolangServer::construct_expr(address, lookup_tbl, symtab, fnc_map, ns);
                     for expp in args {
                         SolangServer::construct_expr(expp, lookup_tbl, symtab, fnc_map, ns);
                     }
-
-                    SolangServer::construct_expr(value, lookup_tbl, symtab, fnc_map, ns);
-                    SolangServer::construct_expr(gas, lookup_tbl, symtab, fnc_map, ns);
+                    if let Some(value) = &call_args.value {
+                        SolangServer::construct_expr(value, lookup_tbl, symtab, fnc_map, ns);
+                    }
+                    if let Some(gas) = &call_args.gas {
+                        SolangServer::construct_expr(gas, lookup_tbl, symtab, fnc_map, ns);
+                    }
                 }
             }
             ast::Expression::ExternalFunctionCallRaw {
-                loc: _,
-                ty: _,
                 address,
                 args,
-                value,
-                gas,
+                call_args,
+                ..
             } => {
                 SolangServer::construct_expr(args, lookup_tbl, symtab, fnc_map, ns);
                 SolangServer::construct_expr(address, lookup_tbl, symtab, fnc_map, ns);
-                SolangServer::construct_expr(value, lookup_tbl, symtab, fnc_map, ns);
-                SolangServer::construct_expr(gas, lookup_tbl, symtab, fnc_map, ns);
+                if let Some(value) = &call_args.value {
+                    SolangServer::construct_expr(value, lookup_tbl, symtab, fnc_map, ns);
+                }
+                if let Some(gas) = &call_args.gas {
+                    SolangServer::construct_expr(gas, lookup_tbl, symtab, fnc_map, ns);
+                }
             }
             ast::Expression::Constructor {
                 loc: _,
                 contract_no: _,
                 constructor_no: _,
                 args,
-                gas,
-                value,
-                salt,
-                space,
+                call_args,
             } => {
-                SolangServer::construct_expr(gas, lookup_tbl, symtab, fnc_map, ns);
+                if let Some(gas) = &call_args.gas {
+                    SolangServer::construct_expr(gas, lookup_tbl, symtab, fnc_map, ns);
+                }
                 for expp in args {
                     SolangServer::construct_expr(expp, lookup_tbl, symtab, fnc_map, ns);
                 }
-                if let Some(optval) = value {
+                if let Some(optval) = &call_args.value {
                     SolangServer::construct_expr(optval, lookup_tbl, symtab, fnc_map, ns);
                 }
-                if let Some(optsalt) = salt {
+                if let Some(optsalt) = &call_args.salt {
                     SolangServer::construct_expr(optsalt, lookup_tbl, symtab, fnc_map, ns);
                 }
-                if let Some(space) = space {
+                if let Some(space) = &call_args.space {
                     SolangServer::construct_expr(space, lookup_tbl, symtab, fnc_map, ns);
                 }
             }
-
-            // Hash table operation expression
-            ast::Expression::Keccak256(_locs, _typ, expr) => {
-                for expp in expr {
-                    SolangServer::construct_expr(expp, lookup_tbl, symtab, fnc_map, ns);
-                }
-                lookup_tbl.push((_locs.1, _locs.2, String::from("Keccak256 hash")));
-            }
-
-            ast::Expression::ReturnData(locs) => {
-                let msg = String::from("Return");
-                lookup_tbl.push((locs.1, locs.2, msg));
-            }
             ast::Expression::Builtin(_locs, _typ, _builtin, expr) => {
                 let msg = SolangServer::construct_builtins(_builtin, ns);
-                lookup_tbl.push((_locs.1, _locs.2, msg));
+                lookup_tbl.push((_locs.start(), _locs.end(), msg));
                 for expp in expr {
                     SolangServer::construct_expr(expp, lookup_tbl, symtab, fnc_map, ns);
                 }
@@ -827,7 +827,7 @@ impl SolangServer {
     ) {
         let msg_typ = SolangServer::expanded_ty(&contvar.ty, ns);
         let msg = format!("{} {}", msg_typ, contvar.name);
-        lookup_tbl.push((contvar.loc.1, contvar.loc.2, msg));
+        lookup_tbl.push((contvar.loc.start(), contvar.loc.end(), msg));
         if let Some(expr) = &contvar.initializer {
             SolangServer::construct_expr(expr, lookup_tbl, samptb, fnc_map, ns);
         }
@@ -840,8 +840,8 @@ impl SolangServer {
         ns: &ast::Namespace,
     ) {
         let msg_typ = &strfld.ty.to_string(ns);
-        let msg = format!("{} {}", msg_typ, strfld.name);
-        lookup_tbl.push((strfld.loc.1, strfld.loc.2, msg));
+        let msg = format!("{} {}", msg_typ, strfld.name_as_str());
+        lookup_tbl.push((strfld.loc.start(), strfld.loc.end(), msg));
     }
 
     // Traverses namespace to build messages stored in the lookup table for hover feature.
@@ -853,31 +853,38 @@ impl SolangServer {
         for enm in &ns.enums {
             for (nam, vals) in &enm.values {
                 let evnt_msg = format!("{} {}, \n\n", nam, vals.1);
-                lookup_tbl.push((vals.0 .1, vals.0 .2, evnt_msg));
+                lookup_tbl.push((vals.0.start(), vals.0.end(), evnt_msg));
             }
 
             let msg_tg = render(&enm.tags[..]);
-            lookup_tbl.push((enm.loc.1, (enm.loc.1 + enm.name.len()), msg_tg));
+            lookup_tbl.push((enm.loc.start(), (enm.loc.start() + enm.name.len()), msg_tg));
         }
 
         for strct in &ns.structs {
-            for filds in &strct.fields {
-                SolangServer::construct_strct(filds, lookup_tbl, ns);
-            }
+            if let pt::Loc::File(_, start, _) = &strct.loc {
+                for filds in &strct.fields {
+                    SolangServer::construct_strct(filds, lookup_tbl, ns);
+                }
 
-            let msg_tg = render(&strct.tags[..]);
-            lookup_tbl.push((strct.loc.1, (strct.loc.1 + strct.name.len()), msg_tg));
+                let msg_tg = render(&strct.tags[..]);
+                lookup_tbl.push((*start, start + strct.name.len(), msg_tg));
+            }
         }
 
         for fnc in &ns.functions {
-            for parm in &fnc.params {
-                let msg = SolangServer::expanded_ty(&parm.ty, ns);
-                lookup_tbl.push((parm.loc.1, parm.loc.2, msg));
+            if fnc.is_accessor {
+                // accessor functions are synthetic; ignore them, all the locations are fake
+                continue;
             }
 
-            for ret in &fnc.returns {
+            for parm in &*fnc.params {
+                let msg = SolangServer::expanded_ty(&parm.ty, ns);
+                lookup_tbl.push((parm.loc.start(), parm.loc.end(), msg));
+            }
+
+            for ret in &*fnc.returns {
                 let msg = SolangServer::expanded_ty(&ret.ty, ns);
-                lookup_tbl.push((ret.loc.1, ret.loc.2, msg));
+                lookup_tbl.push((ret.loc.start(), ret.loc.end(), msg));
             }
 
             for stmt in &fnc.body {
@@ -891,15 +898,19 @@ impl SolangServer {
 
             let msg_tg = render(&constant.tags[..]);
             lookup_tbl.push((
-                constant.loc.1,
-                (constant.loc.1 + constant.name.len()),
+                constant.loc.start(),
+                (constant.loc.start() + constant.name.len()),
                 msg_tg,
             ));
         }
 
         for contrct in &ns.contracts {
             let msg_tg = render(&contrct.tags[..]);
-            lookup_tbl.push((contrct.loc.1, (contrct.loc.1 + msg_tg.len()), msg_tg));
+            lookup_tbl.push((
+                contrct.loc.start(),
+                (contrct.loc.start() + msg_tg.len()),
+                msg_tg,
+            ));
 
             for varscont in &contrct.variables {
                 let samptb = symtable::Symtable::new();
@@ -907,8 +918,8 @@ impl SolangServer {
 
                 let msg_tg = render(&varscont.tags[..]);
                 lookup_tbl.push((
-                    varscont.loc.1,
-                    (varscont.loc.1 + varscont.name.len()),
+                    varscont.loc.start(),
+                    (varscont.loc.start() + varscont.name.len()),
                     msg_tg,
                 ));
             }
@@ -919,11 +930,18 @@ impl SolangServer {
                 SolangServer::construct_strct(filds, lookup_tbl, ns);
             }
             let msg_tg = render(&entdcl.tags[..]);
-            lookup_tbl.push((entdcl.loc.1, (entdcl.loc.1 + entdcl.name.len()), msg_tg));
+            lookup_tbl.push((
+                entdcl.loc.start(),
+                (entdcl.loc.start() + entdcl.name.len()),
+                msg_tg,
+            ));
         }
 
         for lookup in lookup_tbl.iter_mut() {
-            if let Some(msg) = ns.hover_overrides.get(&pt::Loc(0, lookup.0, lookup.1)) {
+            if let Some(msg) = ns
+                .hover_overrides
+                .get(&pt::Loc::File(0, lookup.0, lookup.1))
+            {
                 lookup.2 = msg.clone();
             }
         }
@@ -946,7 +964,7 @@ impl SolangServer {
                     msg.push_str(&format!(
                         "\t{} {}{}\n",
                         field.ty.to_string(ns),
-                        field.name,
+                        field.name_as_str(),
                         if iter.peek().is_some() { "," } else { "" }
                     ));
                 }
@@ -996,7 +1014,7 @@ impl LanguageServer for SolangServer {
             server_info: None,
             capabilities: ServerCapabilities {
                 text_document_sync: Some(TextDocumentSyncCapability::Kind(
-                    TextDocumentSyncKind::Incremental,
+                    TextDocumentSyncKind::INCREMENTAL,
                 )),
                 hover_provider: Some(HoverProviderCapability::Simple(true)),
                 completion_provider: Some(CompletionOptions {
@@ -1031,7 +1049,7 @@ impl LanguageServer for SolangServer {
     async fn initialized(&self, _: InitializedParams) {
         self.client
             .log_message(
-                MessageType::Info,
+                MessageType::INFO,
                 format!("solang language server {} initialized", env!("GIT_HASH")),
             )
             .await;
@@ -1043,25 +1061,25 @@ impl LanguageServer for SolangServer {
 
     async fn did_change_workspace_folders(&self, _: DidChangeWorkspaceFoldersParams) {
         self.client
-            .log_message(MessageType::Info, "workspace folders changed!")
+            .log_message(MessageType::INFO, "workspace folders changed!")
             .await;
     }
 
     async fn did_change_configuration(&self, _: DidChangeConfigurationParams) {
         self.client
-            .log_message(MessageType::Info, "configuration changed!")
+            .log_message(MessageType::INFO, "configuration changed!")
             .await;
     }
 
     async fn did_change_watched_files(&self, _: DidChangeWatchedFilesParams) {
         self.client
-            .log_message(MessageType::Info, "watched files have changed!")
+            .log_message(MessageType::INFO, "watched files have changed!")
             .await;
     }
 
     async fn execute_command(&self, _: ExecuteCommandParams) -> Result<Option<Value>> {
         self.client
-            .log_message(MessageType::Info, "command executed!")
+            .log_message(MessageType::INFO, "command executed!")
             .await;
         Ok(None)
     }
@@ -1119,7 +1137,7 @@ impl LanguageServer for SolangServer {
                     }
                 }) {
                     let msg = &hovers.lookup[pos];
-                    let loc = pt::Loc(0, msg.0, msg.1);
+                    let loc = pt::Loc::File(0, msg.0, msg.1);
                     let range = SolangServer::loc_to_range(&loc, &hovers.file);
 
                     return Ok(Some(Hover {
