@@ -1,15 +1,15 @@
-use clap::{App, Arg, ArgMatches};
+use clap::{Arg, ArgMatches, Command};
 use itertools::Itertools;
 use num_traits::cast::ToPrimitive;
 use serde::Serialize;
 use std::collections::HashMap;
-use std::ffi::OsString;
-use std::fs::File;
+use std::ffi::{OsStr, OsString};
+use std::fs::{create_dir_all, File};
 use std::io::prelude::*;
 use std::path::{Path, PathBuf};
 
 use solang::abi;
-use solang::codegen::{codegen, Options};
+use solang::codegen::{codegen, OptimizationLevel, Options};
 use solang::emit::Generate;
 use solang::file_resolver::FileResolver;
 use solang::sema::{ast::Namespace, diagnostics};
@@ -41,34 +41,35 @@ pub struct JsonResult {
 }
 
 fn main() {
-    let matches = App::new("solang")
+    let matches = Command::new("solang")
         .version(&*format!("version {}", env!("GIT_HASH")))
         .author(env!("CARGO_PKG_AUTHORS"))
         .about(env!("CARGO_PKG_DESCRIPTION"))
         .arg(
-            Arg::with_name("INPUT")
+            Arg::new("INPUT")
                 .help("Solidity input files")
                 .required(true)
                 .conflicts_with("LANGUAGESERVER")
-                .multiple(true),
+                .allow_invalid_utf8(true)
+                .multiple_values(true),
         )
         .arg(
-            Arg::with_name("EMIT")
+            Arg::new("EMIT")
                 .help("Emit compiler state at early stage")
                 .long("emit")
                 .takes_value(true)
-                .possible_values(&["ast", "cfg", "llvm-ir", "llvm-bc", "object", "asm"]),
+                .possible_values(&["ast-dot", "cfg", "llvm-ir", "llvm-bc", "object", "asm"]),
         )
         .arg(
-            Arg::with_name("OPT")
+            Arg::new("OPT")
                 .help("Set llvm optimizer level")
-                .short("O")
+                .short('O')
                 .takes_value(true)
                 .possible_values(&["none", "less", "default", "aggressive"])
                 .default_value("default"),
         )
         .arg(
-            Arg::with_name("TARGET")
+            Arg::new("TARGET")
                 .help("Target to build for")
                 .long("target")
                 .takes_value(true)
@@ -76,94 +77,99 @@ fn main() {
                 .required(true),
         )
         .arg(
-            Arg::with_name("ADDRESS_LENGTH")
+            Arg::new("ADDRESS_LENGTH")
                 .help("Address length on Substrate")
                 .long("address-length")
                 .takes_value(true)
                 .default_value("32"),
         )
         .arg(
-            Arg::with_name("VALUE_LENGTH")
+            Arg::new("VALUE_LENGTH")
                 .help("Value length on Substrate")
                 .long("value-length")
                 .takes_value(true)
                 .default_value("16"),
         )
         .arg(
-            Arg::with_name("STD-JSON")
+            Arg::new("STD-JSON")
                 .help("mimic solidity json output on stdout")
                 .conflicts_with_all(&["VERBOSE", "OUTPUT", "EMIT"])
                 .long("standard-json"),
         )
         .arg(
-            Arg::with_name("VERBOSE")
+            Arg::new("VERBOSE")
                 .help("show debug messages")
-                .short("v")
+                .short('v')
                 .long("verbose"),
         )
         .arg(
-            Arg::with_name("OUTPUT")
+            Arg::new("OUTPUT")
                 .help("output directory")
-                .short("o")
+                .short('o')
                 .long("output")
                 .takes_value(true),
         )
         .arg(
-            Arg::with_name("IMPORTPATH")
+            Arg::new("IMPORTPATH")
                 .help("Directory to search for solidity files")
-                .short("I")
+                .short('I')
                 .long("importpath")
                 .takes_value(true)
-                .multiple(true)
-                .require_delimiter(true),
+                .allow_invalid_utf8(true)
+                .multiple_occurrences(true),
         )
         .arg(
-            Arg::with_name("IMPORTMAP")
+            Arg::new("IMPORTMAP")
                 .help("Map directory to search for solidity files [format: map=path]")
-                .short("m")
+                .short('m')
                 .long("importmap")
                 .takes_value(true)
-                .multiple(true)
-                .require_delimiter(true),
+                .multiple_occurrences(true),
         )
         .arg(
-            Arg::with_name("CONSTANTFOLDING")
+            Arg::new("CONSTANTFOLDING")
                 .help("Disable constant folding codegen optimization")
                 .long("no-constant-folding")
                 .display_order(1),
         )
         .arg(
-            Arg::with_name("STRENGTHREDUCE")
+            Arg::new("STRENGTHREDUCE")
                 .help("Disable strength reduce codegen optimization")
                 .long("no-strength-reduce")
                 .display_order(2),
         )
         .arg(
-            Arg::with_name("DEADSTORAGE")
+            Arg::new("DEADSTORAGE")
                 .help("Disable dead storage codegen optimization")
                 .long("no-dead-storage")
                 .display_order(3),
         )
         .arg(
-            Arg::with_name("VECTORTOSLICE")
+            Arg::new("VECTORTOSLICE")
                 .help("Disable vector to slice codegen optimization")
                 .long("no-vector-to-slice")
                 .display_order(4),
         )
         .arg(
-            Arg::with_name("MATHOVERFLOW")
-                .help("Enable math overflow checking")
-                .long("math-overflow")
+            Arg::new("COMMONSUBEXPRESSIONELIMINATION")
+                .help("Disable common subexpression elimination")
+                .long("no-cse")
                 .display_order(5),
         )
         .arg(
-            Arg::with_name("LANGUAGESERVER")
+            Arg::new("MATHOVERFLOW")
+                .help("Enable math overflow checking")
+                .long("math-overflow")
+                .display_order(6),
+        )
+        .arg(
+            Arg::new("LANGUAGESERVER")
                 .help("Start language server on stdin/stdout")
                 .conflicts_with_all(&["STD-JSON", "OUTPUT", "EMIT", "OPT", "INPUT"])
                 .long("language-server"),
         )
         .arg(
-            Arg::with_name("DOC")
+            Arg::new("DOC")
                 .help("Generate documention for contracts using doc comments")
                 .long("doc"),
         )
@@ -174,7 +180,7 @@ fn main() {
     let address_length = match address_length.parse() {
         Ok(len) if (4..1024).contains(&len) => len,
         _ => {
-            eprintln!("error: address length ‘{}’ is not valid", address_length);
+            eprintln!("error: address length '{}' is not valid", address_length);
             std::process::exit(1);
         }
     };
@@ -184,7 +190,7 @@ fn main() {
     let value_length = match value_length.parse() {
         Ok(len) if (4..1024).contains(&len) => len,
         _ => {
-            eprintln!("error: value length ‘{}’ is not valid", value_length);
+            eprintln!("error: value length '{}' is not valid", value_length);
             std::process::exit(1);
         }
     };
@@ -201,7 +207,7 @@ fn main() {
 
     if !target.is_substrate() && matches.occurrences_of("ADDRESS_LENGTH") > 0 {
         eprintln!(
-            "error: address length cannot be modified for target ‘{}’",
+            "error: address length cannot be modified for target '{}'",
             target
         );
         std::process::exit(1);
@@ -209,14 +215,14 @@ fn main() {
 
     if !target.is_substrate() && matches.occurrences_of("VALUE_LENGTH") > 0 {
         eprintln!(
-            "error: value length cannot be modified for target ‘{}’",
+            "error: value length cannot be modified for target '{}'",
             target
         );
         std::process::exit(1);
     }
 
     if matches.is_present("LANGUAGESERVER") {
-        languageserver::start_server(target);
+        languageserver::start_server(target, matches);
     }
 
     let verbose = matches.is_present("VERBOSE");
@@ -235,7 +241,7 @@ fn main() {
 
     let mut resolver = FileResolver::new();
 
-    for filename in matches.values_of("INPUT").unwrap() {
+    for filename in matches.values_of_os("INPUT").unwrap() {
         if let Ok(path) = PathBuf::from(filename).canonicalize() {
             let _ = resolver.add_import_path(path.parent().unwrap().to_path_buf());
         }
@@ -246,10 +252,10 @@ fn main() {
         std::process::exit(1);
     }
 
-    if let Some(paths) = matches.values_of("IMPORTPATH") {
+    if let Some(paths) = matches.values_of_os("IMPORTPATH") {
         for path in paths {
             if let Err(e) = resolver.add_import_path(PathBuf::from(path)) {
-                eprintln!("error: import path ‘{}’: {}", path, e);
+                eprintln!("error: import path '{}': {}", path.to_string_lossy(), e);
                 std::process::exit(1);
             }
         }
@@ -259,11 +265,11 @@ fn main() {
         for p in maps {
             if let Some((map, path)) = p.split_once('=') {
                 if let Err(e) = resolver.add_import_map(OsString::from(map), PathBuf::from(path)) {
-                    eprintln!("error: import path ‘{}’: {}", path, e);
+                    eprintln!("error: import path '{}': {}", path, e);
                     std::process::exit(1);
                 }
             } else {
-                eprintln!("error: import map ‘{}’: contains no ‘=’", p);
+                eprintln!("error: import map '{}': contains no '='", p);
                 std::process::exit(1);
             }
         }
@@ -274,15 +280,15 @@ fn main() {
         let mut success = true;
         let mut files = Vec::new();
 
-        for filename in matches.values_of("INPUT").unwrap() {
+        for filename in matches.values_of_os("INPUT").unwrap() {
             let ns = solang::parse_and_resolve(filename, &mut resolver, target);
 
-            diagnostics::print_messages(&resolver, &ns, verbose);
+            ns.print_diagnostics(&resolver, verbose);
 
             if ns.contracts.is_empty() {
-                eprintln!("{}: error: no contracts found", filename);
+                eprintln!("{}: error: no contracts found", filename.to_string_lossy());
                 success = false;
-            } else if diagnostics::any_errors(&ns.diagnostics) {
+            } else if ns.diagnostics.any_errors() {
                 success = false;
             } else {
                 files.push(ns);
@@ -295,19 +301,20 @@ fn main() {
         }
     } else {
         let opt_level = match matches.value_of("OPT").unwrap() {
-            "none" => inkwell::OptimizationLevel::None,
-            "less" => inkwell::OptimizationLevel::Less,
-            "default" => inkwell::OptimizationLevel::Default,
-            "aggressive" => inkwell::OptimizationLevel::Aggressive,
+            "none" => OptimizationLevel::None,
+            "less" => OptimizationLevel::Less,
+            "default" => OptimizationLevel::Default,
+            "aggressive" => OptimizationLevel::Aggressive,
             _ => unreachable!(),
         };
 
         let opt = Options {
             dead_storage: !matches.is_present("DEADSTORAGE"),
-            strength_reduce: !matches.is_present("STRENGTHREDUCE"),
             constant_folding: !matches.is_present("CONSTANTFOLDING"),
+            strength_reduce: !matches.is_present("STRENGTHREDUCE"),
             vector_to_slice: !matches.is_present("VECTORTOSLICE"),
             math_overflow_check,
+            common_subexpression_elimination: !matches.is_present("COMMONSUBEXPRESSIONELIMINATION"),
             opt_level,
         };
 
@@ -315,7 +322,7 @@ fn main() {
 
         let mut errors = false;
 
-        for filename in matches.values_of("INPUT").unwrap() {
+        for filename in matches.values_of_os("INPUT").unwrap() {
             match process_file(filename, &mut resolver, target, &matches, &mut json, &opt) {
                 Ok(ns) => namespaces.push(ns),
                 Err(_) => {
@@ -324,9 +331,18 @@ fn main() {
             }
         }
 
+        if let Some("ast-dot") = matches.value_of("EMIT") {
+            std::process::exit(0);
+        }
+
         if errors {
-            eprintln!("error: not all contracts are valid");
-            std::process::exit(1);
+            if matches.is_present("STD-JSON") {
+                println!("{}", serde_json::to_string(&json).unwrap());
+                std::process::exit(0);
+            } else {
+                eprintln!("error: not all contracts are valid");
+                std::process::exit(1);
+            }
         }
 
         if target == solang::Target::Solana {
@@ -336,7 +352,7 @@ fn main() {
                 &context,
                 &namespaces,
                 "bundle.sol",
-                opt_level,
+                opt_level.into(),
                 math_overflow_check,
             );
 
@@ -371,17 +387,7 @@ fn main() {
                 if matches.is_present("STD-JSON") {
                     json.program = hex::encode_upper(&code);
                 } else {
-                    let mut file = match File::create(&bin_filename) {
-                        Ok(file) => file,
-                        Err(err) => {
-                            eprintln!(
-                                "error: cannot create file ‘{}’: {}",
-                                bin_filename.display(),
-                                err,
-                            );
-                            std::process::exit(1);
-                        }
-                    };
+                    let mut file = create_file(&bin_filename);
                     file.write_all(&code).unwrap();
 
                     // Write all ABI files
@@ -405,17 +411,7 @@ fn main() {
                                 );
                             }
 
-                            let mut file = match File::create(abi_filename) {
-                                Ok(file) => file,
-                                Err(err) => {
-                                    eprintln!(
-                                        "error: cannot create file ‘{}’: {}",
-                                        bin_filename.display(),
-                                        err
-                                    );
-                                    std::process::exit(1);
-                                }
-                            };
+                            let mut file = create_file(&abi_filename);
 
                             file.write_all(abi_bytes.as_bytes()).unwrap();
                         }
@@ -435,7 +431,7 @@ fn output_file(matches: &ArgMatches, stem: &str, ext: &str) -> PathBuf {
 }
 
 fn process_file(
-    filename: &str,
+    filename: &OsStr,
     resolver: &mut FileResolver,
     target: solang::Target,
     matches: &ArgMatches,
@@ -453,21 +449,37 @@ fn process_file(
     codegen(&mut ns, opt);
 
     if matches.is_present("STD-JSON") {
-        let mut out = diagnostics::message_as_json(&ns, resolver);
+        let mut out = ns.diagnostics_as_json(resolver);
         json.errors.append(&mut out);
     } else {
-        diagnostics::print_messages(resolver, &ns, verbose);
+        ns.print_diagnostics(resolver, verbose);
     }
 
-    if ns.contracts.is_empty() || diagnostics::any_errors(&ns.diagnostics) {
-        return Err(());
-    }
+    if let Some("ast-dot") = matches.value_of("EMIT") {
+        let filepath = PathBuf::from(filename);
+        let stem = filepath.file_stem().unwrap().to_string_lossy();
+        let dot_filename = output_file(matches, &stem, "dot");
+
+        if verbose {
+            eprintln!("info: Saving graphviz dot {}", dot_filename.display());
+        }
+
+        let dot = ns.dotgraphviz();
+
+        let mut file = create_file(&dot_filename);
+
+        if let Err(err) = file.write_all(dot.as_bytes()) {
+            eprintln!("{}: error: {}", dot_filename.display(), err);
+            std::process::exit(1);
+        }
 
-    if let Some("ast") = matches.value_of("EMIT") {
-        println!("{}", ns.print(filename));
         return Ok(ns);
     }
 
+    if ns.contracts.is_empty() || ns.diagnostics.any_errors() {
+        return Err(());
+    }
+
     // emit phase
     for contract_no in 0..ns.contracts.len() {
         let resolved_contract = &ns.contracts[contract_no];
@@ -511,12 +523,13 @@ fn process_file(
         }
 
         let context = inkwell::context::Context::create();
+        let filename_string = filename.to_string_lossy();
 
         let binary = resolved_contract.emit(
             &ns,
             &context,
-            filename,
-            opt.opt_level,
+            &filename_string,
+            opt.opt_level.into(),
             opt.math_overflow_check,
         );
 
@@ -546,17 +559,7 @@ fn process_file(
                 );
             }
 
-            let mut file = match File::create(&bin_filename) {
-                Ok(file) => file,
-                Err(err) => {
-                    eprintln!(
-                        "error: cannot create file ‘{}’: {}",
-                        bin_filename.display(),
-                        err,
-                    );
-                    std::process::exit(1);
-                }
-            };
+            let mut file = create_file(&bin_filename);
             file.write_all(&resolved_contract.code).unwrap();
 
             let (abi_bytes, abi_ext) =
@@ -571,22 +574,13 @@ fn process_file(
                 );
             }
 
-            let mut file = match File::create(&abi_filename) {
-                Ok(file) => file,
-                Err(err) => {
-                    eprintln!(
-                        "error: cannot create file ‘{}’: {}",
-                        abi_filename.display(),
-                        err,
-                    );
-                    std::process::exit(1);
-                }
-            };
+            let mut file = create_file(&abi_filename);
             file.write_all(abi_bytes.as_bytes()).unwrap();
         }
     }
 
-    json.contracts.insert(filename.to_owned(), json_contracts);
+    json.contracts
+        .insert(filename.to_string_lossy().to_string(), json_contracts);
 
     Ok(ns)
 }
@@ -702,7 +696,7 @@ fn save_intermediates(binary: &solang::emit::Binary, matches: &ArgMatches) -> bo
                 );
             }
 
-            let mut file = File::create(obj_filename).unwrap();
+            let mut file = create_file(&obj_filename);
             file.write_all(&obj).unwrap();
             true
         }
@@ -725,12 +719,33 @@ fn save_intermediates(binary: &solang::emit::Binary, matches: &ArgMatches) -> bo
                 );
             }
 
-            let mut file = File::create(obj_filename).unwrap();
+            let mut file = create_file(&obj_filename);
             file.write_all(&obj).unwrap();
             true
         }
         Some("cfg") => true,
-        Some("ast") => true,
+        Some("ast-dot") => true,
         _ => false,
     }
 }
+
+fn create_file(path: &Path) -> File {
+    if let Some(parent) = path.parent() {
+        if let Err(err) = create_dir_all(parent) {
+            eprintln!(
+                "error: cannot create output directory '{}': {}",
+                parent.display(),
+                err
+            );
+            std::process::exit(1);
+        }
+    }
+
+    match File::create(path) {
+        Ok(file) => file,
+        Err(err) => {
+            eprintln!("error: cannot create file '{}': {}", path.display(), err,);
+            std::process::exit(1);
+        }
+    }
+}
