@@ -1,22 +1,26 @@
-use num_bigint::BigInt;
-use std::collections::{BTreeSet, HashMap, HashSet};
+use indexmap::IndexMap;
+use std::collections::{BTreeMap, BTreeSet, HashMap, HashSet};
 use std::fmt;
 use std::str;
+use std::sync::Arc;
 
 use super::statements::{statement, LoopScopes};
 use super::{
-    constant_folding, dead_storage, expression::expression, reaching_definitions, strength_reduce,
+    constant_folding, dead_storage,
+    expression::expression,
+    reaching_definitions, strength_reduce,
+    vartable::{Vars, Vartable},
     vector_to_slice, Options,
 };
-use crate::codegen::undefined_variable;
+use crate::codegen::subexpression_elimination::common_sub_expression_elimination;
+use crate::codegen::{undefined_variable, Expression, LLVMName};
 use crate::parser::pt;
-use crate::sema::ast::{
-    CallTy, Contract, Expression, Function, Namespace, Parameter, StringLocation, Type,
-};
+use crate::parser::pt::CodeLocation;
+use crate::sema::ast::RetrieveType;
+use crate::sema::ast::{CallTy, Contract, Function, Namespace, Parameter, StringLocation, Type};
 use crate::sema::contracts::{collect_base_args, visit_bases};
-use crate::sema::symtable::Symtable;
-
-pub type Vars = HashMap<usize, Variable>;
+use crate::sema::Recurse;
+use crate::{ast, Target};
 
 #[derive(Clone)]
 #[allow(clippy::large_enum_variant)]
@@ -71,15 +75,16 @@ pub enum Instr {
         storage: Expression,
         offset: Expression,
     },
-    /// Push an element onto a bytes in storage
+    /// Push an element onto an array in storage
     PushStorage {
         res: usize,
-        value: Expression,
+        ty: Type,
+        value: Option<Expression>,
         storage: Expression,
     },
-    /// Pop an element from a bytes in storage
+    /// Pop an element from an array in storage
     PopStorage {
-        res: usize,
+        res: Option<usize>,
         ty: Type,
         storage: Expression,
     },
@@ -111,6 +116,7 @@ pub enum Instr {
     ExternalCall {
         success: Option<usize>,
         address: Option<Expression>,
+        accounts: Option<Expression>,
         payload: Expression,
         value: Expression,
         gas: Expression,
@@ -143,6 +149,12 @@ pub enum Instr {
         topics: Vec<Expression>,
         topic_tys: Vec<Type>,
     },
+    /// Write Buffer
+    WriteBuffer {
+        buf: Expression,
+        offset: Expression,
+        value: Expression,
+    },
     /// Do nothing
     Nop,
 }
@@ -171,11 +183,16 @@ impl Instr {
                 expr.recurse(cx, f);
             }
 
-            Instr::SetStorage { value, storage, .. }
-            | Instr::PushStorage { value, storage, .. } => {
+            Instr::SetStorage { value, storage, .. } => {
                 value.recurse(cx, f);
                 storage.recurse(cx, f);
             }
+            Instr::PushStorage { value, storage, .. } => {
+                if let Some(value) = value {
+                    value.recurse(cx, f);
+                }
+                storage.recurse(cx, f);
+            }
 
             Instr::SetStorageBytes {
                 value,
@@ -248,6 +265,11 @@ impl Instr {
                 }
             }
 
+            Instr::WriteBuffer { offset, value, .. } => {
+                value.recurse(cx, f);
+                offset.recurse(cx, f);
+            }
+
             Instr::AssertFailure { expr: None }
             | Instr::Unreachable
             | Instr::Nop
@@ -291,6 +313,7 @@ pub struct BasicBlock {
     pub name: String,
     pub instr: Vec<Instr>,
     pub defs: reaching_definitions::VarDefs,
+    pub loop_reaching_variables: HashSet<usize>,
     pub transfers: Vec<Vec<reaching_definitions::Transfer>>,
 }
 
@@ -303,9 +326,9 @@ impl BasicBlock {
 #[derive(Clone)]
 pub struct ControlFlowGraph {
     pub name: String,
-    pub function_no: Option<usize>,
-    pub params: Vec<Parameter>,
-    pub returns: Vec<Parameter>,
+    pub function_no: ASTFunction,
+    pub params: Arc<Vec<Parameter>>,
+    pub returns: Arc<Vec<Parameter>>,
     pub vars: Vars,
     pub blocks: Vec<BasicBlock>,
     pub nonpayable: bool,
@@ -315,14 +338,21 @@ pub struct ControlFlowGraph {
     current: usize,
 }
 
+#[derive(Clone)]
+pub enum ASTFunction {
+    SolidityFunction(usize),
+    YulFunction(usize),
+    None,
+}
+
 impl ControlFlowGraph {
-    pub fn new(name: String, function_no: Option<usize>) -> Self {
+    pub fn new(name: String, function_no: ASTFunction) -> Self {
         let mut cfg = ControlFlowGraph {
             name,
             function_no,
-            params: Vec::new(),
-            returns: Vec::new(),
-            vars: HashMap::new(),
+            params: Arc::new(Vec::new()),
+            returns: Arc::new(Vec::new()),
+            vars: IndexMap::new(),
             blocks: Vec::new(),
             nonpayable: false,
             public: false,
@@ -340,10 +370,10 @@ impl ControlFlowGraph {
     pub fn placeholder() -> Self {
         ControlFlowGraph {
             name: String::new(),
-            function_no: None,
-            params: Vec::new(),
-            returns: Vec::new(),
-            vars: HashMap::new(),
+            function_no: ASTFunction::None,
+            params: Arc::new(Vec::new()),
+            returns: Arc::new(Vec::new()),
+            vars: IndexMap::new(),
             blocks: Vec::new(),
             nonpayable: false,
             public: false,
@@ -366,7 +396,8 @@ impl ControlFlowGraph {
             instr: Vec::new(),
             phis: None,
             transfers: Vec::new(),
-            defs: HashMap::new(),
+            defs: IndexMap::new(),
+            loop_reaching_variables: HashSet::new(),
         });
 
         pos
@@ -398,8 +429,11 @@ impl ControlFlowGraph {
                 format!("{}", String::from_utf8_lossy(s))
             }
             Expression::BytesLiteral(_, _, s) => format!("hex\"{}\"", hex::encode(s)),
+            Expression::NumberLiteral(_, ty @ Type::Address(_), n) => {
+                format!("{} {:#x}", ty.to_string(ns), n)
+            }
             Expression::NumberLiteral(_, ty, n) => {
-                format!("{} {}", ty.to_string(ns), n.to_str_radix(10))
+                format!("{} {}", ty.to_string(ns), n)
             }
             Expression::RationalNumberLiteral(_, ty, n) => {
                 format!("{} {}", ty.to_string(ns), n)
@@ -485,23 +519,9 @@ impl ControlFlowGraph {
                 self.expr_to_string(contract, ns, r)
             ),
             Expression::Variable(_, _, res) => format!("%{}", self.vars[res].id.name),
-            Expression::ConstantVariable(_, _, Some(var_contract_no), var_no)
-            | Expression::StorageVariable(_, _, var_contract_no, var_no) => format!(
-                "${}.{}",
-                ns.contracts[*var_contract_no].name,
-                ns.contracts[*var_contract_no].variables[*var_no].name
-            ),
-            Expression::ConstantVariable(_, _, None, var_no) => {
-                format!("${}", ns.constants[*var_no].name)
-            }
             Expression::Load(_, _, expr) => {
                 format!("(load {})", self.expr_to_string(contract, ns, expr))
             }
-            Expression::StorageLoad(_, ty, expr) => format!(
-                "(loadstorage ty:{} {})",
-                ty.to_string(ns),
-                self.expr_to_string(contract, ns, expr)
-            ),
             Expression::ZeroExt(_, ty, e) => format!(
                 "(zext {} {})",
                 ty.to_string(ns),
@@ -547,22 +567,12 @@ impl ControlFlowGraph {
                 self.expr_to_string(contract, ns, l),
                 self.expr_to_string(contract, ns, r)
             ),
-            Expression::Subscript(_, ty, a, i) => format!(
+            Expression::Subscript(_, _, ty, a, i) => format!(
                 "(subscript {} {}[{}])",
                 ty.to_string(ns),
                 self.expr_to_string(contract, ns, a),
                 self.expr_to_string(contract, ns, i)
             ),
-            Expression::DynamicArraySubscript(_, _, a, i) => format!(
-                "(darray index {}[{}])",
-                self.expr_to_string(contract, ns, a),
-                self.expr_to_string(contract, ns, i)
-            ),
-            Expression::StorageBytesSubscript(_, a, i) => format!(
-                "(storage bytes index {}[{}])",
-                self.expr_to_string(contract, ns, a),
-                self.expr_to_string(contract, ns, i)
-            ),
             Expression::StorageArrayLength { array, elem_ty, .. } => format!(
                 "(storage array length {}[{}])",
                 self.expr_to_string(contract, ns, array),
@@ -573,22 +583,6 @@ impl ControlFlowGraph {
                 self.expr_to_string(contract, ns, a),
                 f
             ),
-            Expression::Or(_, l, r) => format!(
-                "({} || {})",
-                self.expr_to_string(contract, ns, l),
-                self.expr_to_string(contract, ns, r)
-            ),
-            Expression::And(_, l, r) => format!(
-                "({} && {})",
-                self.expr_to_string(contract, ns, l),
-                self.expr_to_string(contract, ns, r)
-            ),
-            Expression::Ternary(_, _, c, l, r) => format!(
-                "({} ? {} : {})",
-                self.expr_to_string(contract, ns, c),
-                self.expr_to_string(contract, ns, l),
-                self.expr_to_string(contract, ns, r)
-            ),
             Expression::Not(_, e) => format!("!{}", self.expr_to_string(contract, ns, e)),
             Expression::Complement(_, _, e) => format!("~{}", self.expr_to_string(contract, ns, e)),
             Expression::UnaryMinus(_, _, e) => format!("-{}", self.expr_to_string(contract, ns, e)),
@@ -607,9 +601,6 @@ impl ControlFlowGraph {
                     Err(_) => format!("hex\"{}\"", hex::encode(init)),
                 }
             ),
-            Expression::DynamicArrayLength(_, a) => {
-                format!("(darray {} len)", self.expr_to_string(contract, ns, a))
-            }
             Expression::StringCompare(_, l, r) => format!(
                 "(strcmp ({}) ({}))",
                 self.location_to_string(contract, ns, l),
@@ -628,19 +619,6 @@ impl ControlFlowGraph {
                     .collect::<Vec<String>>()
                     .join(", ")
             ),
-            Expression::InternalFunction {
-                function_no,
-                signature,
-                ..
-            } => {
-                let function_no = if let Some(signature) = signature {
-                    contract.virtual_functions[signature]
-                } else {
-                    *function_no
-                };
-
-                ns.functions[function_no].print_name(ns)
-            }
             Expression::ExternalFunction {
                 address,
                 function_no,
@@ -653,41 +631,6 @@ impl ControlFlowGraph {
             Expression::InternalFunctionCfg(cfg_no) => {
                 format!("function {}", contract.cfg[*cfg_no].name)
             }
-            Expression::InternalFunctionCall { function, args, .. } => format!(
-                "(call {} ({})",
-                self.expr_to_string(contract, ns, function),
-                args.iter()
-                    .map(|a| self.expr_to_string(contract, ns, a))
-                    .collect::<Vec<String>>()
-                    .join(", ")
-            ),
-            Expression::Constructor {
-                contract_no,
-                constructor_no: Some(constructor_no),
-                args,
-                ..
-            } => format!(
-                "(constructor:{} ({}) ({})",
-                ns.contracts[*contract_no].name,
-                ns.functions[*constructor_no].signature,
-                args.iter()
-                    .map(|a| self.expr_to_string(contract, ns, a))
-                    .collect::<Vec<String>>()
-                    .join(", ")
-            ),
-            Expression::Constructor {
-                contract_no,
-                constructor_no: None,
-                args,
-                ..
-            } => format!(
-                "(constructor:{} ({})",
-                ns.contracts[*contract_no].name,
-                args.iter()
-                    .map(|a| self.expr_to_string(contract, ns, a))
-                    .collect::<Vec<String>>()
-                    .join(", ")
-            ),
             Expression::CodeLiteral(_, contract_no, runtime) => format!(
                 "({} code contract {})",
                 if *runtime {
@@ -697,32 +640,7 @@ impl ControlFlowGraph {
                 },
                 ns.contracts[*contract_no].name,
             ),
-            Expression::ExternalFunctionCall { function, args, .. } => format!(
-                "(external call {} ({})",
-                self.expr_to_string(contract, ns, function),
-                args.iter()
-                    .map(|a| self.expr_to_string(contract, ns, a))
-                    .collect::<Vec<String>>()
-                    .join(", ")
-            ),
             Expression::ReturnData(_) => "(external call return data)".to_string(),
-            Expression::Assign(_, _, l, r) => format!(
-                "{} = {}",
-                self.expr_to_string(contract, ns, l),
-                self.expr_to_string(contract, ns, r)
-            ),
-            Expression::PostDecrement(_, _, _, e) => {
-                format!("{}--", self.expr_to_string(contract, ns, e),)
-            }
-            Expression::PostIncrement(_, _, _, e) => {
-                format!("{}++", self.expr_to_string(contract, ns, e),)
-            }
-            Expression::PreDecrement(_, _, _, e) => {
-                format!("--{}", self.expr_to_string(contract, ns, e),)
-            }
-            Expression::PreIncrement(_, _, _, e) => {
-                format!("++{}", self.expr_to_string(contract, ns, e),)
-            }
             Expression::Cast(_, ty, e) => format!(
                 "{}({})",
                 ty.to_string(ns),
@@ -762,6 +680,9 @@ impl ControlFlowGraph {
                     .join(", ")
             ),
             Expression::Undefined(_) => "undef".to_string(),
+            Expression::GetRef(_, _, expr) => {
+                format!("(deref {}", self.expr_to_string(contract, ns, expr))
+            }
             _ => panic!("{:?}", expr),
         }
     }
@@ -770,7 +691,7 @@ impl ControlFlowGraph {
         &self,
         contract: &Contract,
         ns: &Namespace,
-        l: &StringLocation,
+        l: &StringLocation<Expression>,
     ) -> String {
         match l {
             StringLocation::RunTime(e) => self.expr_to_string(contract, ns, e),
@@ -837,17 +758,27 @@ impl ControlFlowGraph {
             ),
             Instr::PushStorage {
                 res,
+                ty,
                 storage,
                 value,
             } => {
                 format!(
-                    "%{} = push storage slot({}) = {}",
+                    "%{} = push storage ty:{} slot:{} = {}",
                     self.vars[res].id.name,
+                    ty.to_string(ns),
                     self.expr_to_string(contract, ns, storage),
-                    self.expr_to_string(contract, ns, value),
+                    if let Some(value) = value {
+                        self.expr_to_string(contract, ns, value)
+                    } else {
+                        String::from("empty")
+                    }
                 )
             }
-            Instr::PopStorage { res, ty, storage } => {
+            Instr::PopStorage {
+                res: Some(res),
+                ty,
+                storage,
+            } => {
                 format!(
                     "%{} = pop storage ty:{} slot({})",
                     self.vars[res].id.name,
@@ -855,6 +786,17 @@ impl ControlFlowGraph {
                     self.expr_to_string(contract, ns, storage),
                 )
             }
+            Instr::PopStorage {
+                res: None,
+                ty,
+                storage,
+            } => {
+                format!(
+                    "pop storage ty:{} slot({})",
+                    ty.to_string(ns),
+                    self.expr_to_string(contract, ns, storage),
+                )
+            }
             Instr::PushMemory {
                 res,
                 ty,
@@ -916,11 +858,12 @@ impl ControlFlowGraph {
                 address,
                 payload,
                 value,
+                accounts,
                 gas,
                 callty,
             } => {
                 format!(
-                    "{} = external call::{} address:{} payload:{} value:{} gas:{}",
+                    "{} = external call::{} address:{} payload:{} value:{} gas:{} accounts:{}",
                     match success {
                         Some(i) => format!("%{}", self.vars[i].id.name),
                         None => "_".to_string(),
@@ -934,6 +877,11 @@ impl ControlFlowGraph {
                     self.expr_to_string(contract, ns, payload),
                     self.expr_to_string(contract, ns, value),
                     self.expr_to_string(contract, ns, gas),
+                    if let Some(accounts) = accounts {
+                        self.expr_to_string(contract, ns, accounts)
+                    } else {
+                        String::new()
+                    },
                 )
             }
             Instr::ValueTransfer {
@@ -1026,6 +974,12 @@ impl ControlFlowGraph {
                 "selfdestruct {}",
                 self.expr_to_string(contract, ns, recipient)
             ),
+            Instr::WriteBuffer { buf, offset, value } => format!(
+                "writebuffer buffer:{} offset:{} value:{}",
+                self.expr_to_string(contract, ns, buf),
+                self.expr_to_string(contract, ns, offset),
+                self.expr_to_string(contract, ns, value)
+            ),
             Instr::EmitEvent {
                 data,
                 topics,
@@ -1151,7 +1105,7 @@ pub fn generate_cfg(
         return;
     }
 
-    let mut cfg = function_cfg(contract_no, function_no, ns);
+    let mut cfg = function_cfg(contract_no, function_no, ns, opt);
 
     let default_constructor = &ns.default_constructor(contract_no);
     let func = match function_no {
@@ -1185,6 +1139,7 @@ pub fn generate_cfg(
                 chain_no,
                 args,
                 ns,
+                opt,
             );
 
             cfg = new_cfg;
@@ -1203,11 +1158,11 @@ pub fn generate_cfg(
 
 /// resolve modifier call
 fn resolve_modifier_call<'a>(
-    call: &'a Expression,
+    call: &'a ast::Expression,
     contract: &Contract,
-) -> (usize, &'a Vec<Expression>) {
-    if let Expression::InternalFunctionCall { function, args, .. } = call {
-        if let Expression::InternalFunction {
+) -> (usize, &'a Vec<ast::Expression>) {
+    if let ast::Expression::InternalFunctionCall { function, args, .. } = call {
+        if let ast::Expression::InternalFunction {
             function_no,
             signature,
             ..
@@ -1253,6 +1208,11 @@ pub fn optimize_and_check_cfg(
     if opt.dead_storage {
         dead_storage::dead_storage(cfg, ns);
     }
+
+    // If the function is a default constructor, there is nothing to optimize.
+    if opt.common_subexpression_elimination && func_no.is_some() {
+        common_sub_expression_elimination(cfg, ns);
+    }
 }
 
 /// Generate the CFG for a function. If function_no is None, generate the implicit default
@@ -1261,6 +1221,7 @@ fn function_cfg(
     contract_no: usize,
     function_no: Option<usize>,
     ns: &mut Namespace,
+    opt: &Options,
 ) -> ControlFlowGraph {
     let mut vartab = match function_no {
         Some(function_no) => {
@@ -1297,7 +1258,14 @@ fn function_cfg(
         _ => format!("{}::{}", contract_name, func.ty),
     };
 
-    let mut cfg = ControlFlowGraph::new(name, function_no);
+    let mut cfg = ControlFlowGraph::new(
+        name,
+        if let Some(num) = function_no {
+            ASTFunction::SolidityFunction(num)
+        } else {
+            ASTFunction::None
+        },
+    );
 
     cfg.params = func.params.clone();
     cfg.returns = func.returns.clone();
@@ -1305,8 +1273,8 @@ fn function_cfg(
 
     // a function is public if is not a library and not a base constructor
     cfg.public = if let Some(base_contract_no) = func.contract_no {
-        !ns.contracts[base_contract_no].is_library()
-            && !(func.is_constructor() && contract_no != base_contract_no)
+        !(ns.contracts[base_contract_no].is_library()
+            || func.is_constructor() && contract_no != base_contract_no)
             && func.is_public()
     } else {
         false
@@ -1334,7 +1302,7 @@ fn function_cfg(
             cfg.add(
                 &mut vartab,
                 Instr::Set {
-                    loc: func.params[i].name_loc.unwrap(),
+                    loc: func.params[i].loc,
                     res: *pos,
                     expr: Expression::FunctionArg(var.id.loc, var.ty.clone(), i),
                 },
@@ -1349,8 +1317,8 @@ fn function_cfg(
     // known the top level constructor, since the arguments can be specified elsewhere
     // on a constructor for a superior class
     if func.ty == pt::FunctionTy::Constructor && func.contract_no == Some(contract_no) {
-        let mut all_base_args = HashMap::new();
-        let mut diagnostics = HashSet::new();
+        let mut all_base_args = BTreeMap::new();
+        let mut diagnostics = BTreeSet::new();
 
         // Find all the resolved arguments for base contracts. These can be attached
         // to the contract, or the constructor. Contracts can have multiple constructors
@@ -1401,7 +1369,7 @@ fn function_cfg(
                     .enumerate()
                     .map(|(i, a)| {
                         let expr =
-                            expression(a, &mut cfg, contract_no, Some(func), ns, &mut vartab);
+                            expression(a, &mut cfg, contract_no, Some(func), ns, &mut vartab, opt);
 
                         if let Some(id) = &func.symtable.arguments[i] {
                             let ty = expr.ty();
@@ -1410,7 +1378,7 @@ fn function_cfg(
                             cfg.add(
                                 &mut vartab,
                                 Instr::Set {
-                                    loc: func.params[i].name_loc.unwrap_or(pt::Loc(0, 0, 0)),
+                                    loc: func.params[i].loc,
                                     res: *id,
                                     expr,
                                 },
@@ -1462,12 +1430,12 @@ fn function_cfg(
 
     // named returns should be populated
     for (i, pos) in func.symtable.returns.iter().enumerate() {
-        if !func.returns[i].name.is_empty() {
+        if let Some(name) = &func.returns[i].id {
             if let Some(expr) = func.returns[i].ty.default(ns) {
                 cfg.add(
                     &mut vartab,
                     Instr::Set {
-                        loc: func.returns[i].name_loc.unwrap(),
+                        loc: name.loc,
                         res: *pos,
                         expr,
                     },
@@ -1487,6 +1455,33 @@ fn function_cfg(
             &mut loops,
             None,
             None,
+            opt,
+        );
+    }
+
+    if func
+        .body
+        .last()
+        .map(|stmt| stmt.reachable())
+        .unwrap_or(true)
+    {
+        // add implicit return
+        cfg.add(
+            &mut vartab,
+            Instr::Return {
+                value: func
+                    .symtable
+                    .returns
+                    .iter()
+                    .map(|pos| {
+                        Expression::Variable(
+                            pt::Loc::Codegen,
+                            func.symtable.vars[pos].ty.clone(),
+                            *pos,
+                        )
+                    })
+                    .collect::<Vec<_>>(),
+            },
         );
     }
 
@@ -1505,8 +1500,9 @@ pub fn generate_modifier_dispatch(
     modifier: &Function,
     cfg_no: usize,
     chain_no: usize,
-    args: &[Expression],
+    args: &[ast::Expression],
     ns: &Namespace,
+    opt: &Options,
 ) -> (ControlFlowGraph, usize) {
     let name = format!(
         "{}::{}::{}::modifier{}::{}",
@@ -1516,7 +1512,7 @@ pub fn generate_modifier_dispatch(
         chain_no,
         modifier.llvm_symbol(ns)
     );
-    let mut cfg = ControlFlowGraph::new(name, None);
+    let mut cfg = ControlFlowGraph::new(name, ASTFunction::None);
 
     cfg.params = func.params.clone();
     cfg.returns = func.returns.clone();
@@ -1534,7 +1530,7 @@ pub fn generate_modifier_dispatch(
             cfg.add(
                 &mut vartab,
                 Instr::Set {
-                    loc: pt::Loc(0, 0, 0),
+                    loc: pt::Loc::Codegen,
                     res: *pos,
                     expr: Expression::FunctionArg(var.id.loc, var.ty.clone(), i),
                 },
@@ -1545,11 +1541,19 @@ pub fn generate_modifier_dispatch(
     // now set the modifier args
     for (i, arg) in modifier.symtable.arguments.iter().enumerate() {
         if let Some(pos) = arg {
-            let expr = expression(&args[i], &mut cfg, contract_no, Some(func), ns, &mut vartab);
+            let expr = expression(
+                &args[i],
+                &mut cfg,
+                contract_no,
+                Some(func),
+                ns,
+                &mut vartab,
+                opt,
+            );
             cfg.add(
                 &mut vartab,
                 Instr::Set {
-                    loc: pt::Loc(0, 0, 0),
+                    loc: pt::Loc::Codegen,
                     res: *pos,
                     expr,
                 },
@@ -1597,198 +1601,41 @@ pub fn generate_modifier_dispatch(
             &mut loops,
             Some(&placeholder),
             Some(&return_instr),
+            opt,
         );
     }
 
+    if modifier
+        .body
+        .last()
+        .map(|stmt| stmt.reachable())
+        .unwrap_or(true)
+    {
+        // add implicit return
+        cfg.add(
+            &mut vartab,
+            Instr::Return {
+                value: func
+                    .symtable
+                    .returns
+                    .iter()
+                    .map(|pos| {
+                        Expression::Variable(
+                            pt::Loc::Codegen,
+                            func.symtable.vars[pos].ty.clone(),
+                            *pos,
+                        )
+                    })
+                    .collect::<Vec<_>>(),
+            },
+        );
+    }
     let (vars, next_id) = vartab.drain();
     cfg.vars = vars;
 
     (cfg, next_id)
 }
 
-#[derive(Clone)]
-pub enum Storage {
-    Constant(usize),
-    Contract(BigInt),
-    Local,
-}
-
-#[derive(Clone)]
-pub struct Variable {
-    pub id: pt::Identifier,
-    pub ty: Type,
-    pub storage: Storage,
-}
-
-#[derive(Default)]
-pub struct Vartable {
-    vars: Vars,
-    next_id: usize,
-    dirty: Vec<DirtyTracker>,
-}
-
-pub struct DirtyTracker {
-    lim: usize,
-    set: BTreeSet<usize>,
-}
-
-impl Vartable {
-    pub fn from_symbol_table(sym: &Symtable, next_id: usize) -> Self {
-        let mut vars = HashMap::new();
-
-        for (var_no, v) in &sym.vars {
-            let id = Vartable::make_unique(&vars, &v.id, *var_no);
-
-            vars.insert(
-                *var_no,
-                Variable {
-                    id,
-                    ty: v.ty.clone(),
-                    storage: Storage::Local,
-                },
-            );
-        }
-
-        Vartable {
-            vars,
-            dirty: Vec::new(),
-            next_id,
-        }
-    }
-
-    pub fn add_symbol_table(&mut self, sym: &Symtable) {
-        for (var_no, v) in &sym.vars {
-            let id = Vartable::make_unique(&self.vars, &v.id, *var_no);
-
-            self.vars.insert(
-                *var_no,
-                Variable {
-                    id,
-                    ty: v.ty.clone(),
-                    storage: Storage::Local,
-                },
-            );
-        }
-    }
-
-    fn make_unique(vars: &Vars, id: &pt::Identifier, no: usize) -> pt::Identifier {
-        let mut id = id.clone();
-
-        if vars.iter().any(|(_, var)| var.id.name == id.name) {
-            id.name = format!("{}.{}", id.name, no);
-        }
-
-        id
-    }
-
-    pub fn new(next_id: usize) -> Self {
-        Vartable {
-            vars: HashMap::new(),
-            dirty: Vec::new(),
-            next_id,
-        }
-    }
-
-    pub fn add(&mut self, id: &pt::Identifier, ty: Type) -> Option<usize> {
-        let var_no = self.next_id;
-        self.next_id += 1;
-
-        let id = Vartable::make_unique(&self.vars, id, var_no);
-
-        self.vars.insert(
-            var_no,
-            Variable {
-                id,
-                ty,
-                storage: Storage::Local,
-            },
-        );
-
-        Some(var_no)
-    }
-
-    pub fn temp_anonymous(&mut self, ty: &Type) -> usize {
-        let var_no = self.next_id;
-        self.next_id += 1;
-
-        self.vars.insert(
-            var_no,
-            Variable {
-                id: pt::Identifier {
-                    name: format!("temp.{}", var_no),
-                    loc: pt::Loc(0, 0, 0),
-                },
-                ty: ty.clone(),
-                storage: Storage::Local,
-            },
-        );
-
-        var_no
-    }
-
-    pub fn temp(&mut self, id: &pt::Identifier, ty: &Type) -> usize {
-        let var_no = self.next_id;
-        self.next_id += 1;
-
-        self.vars.insert(
-            var_no,
-            Variable {
-                id: pt::Identifier {
-                    name: format!("{}.temp.{}", id.name, var_no),
-                    loc: id.loc,
-                },
-                ty: ty.clone(),
-                storage: Storage::Local,
-            },
-        );
-
-        var_no
-    }
-
-    pub fn temp_name(&mut self, name: &str, ty: &Type) -> usize {
-        let var_no = self.next_id;
-        self.next_id += 1;
-
-        self.vars.insert(
-            var_no,
-            Variable {
-                id: pt::Identifier {
-                    name: format!("{}.temp.{}", name, var_no),
-                    loc: pt::Loc(0, 0, 0),
-                },
-                ty: ty.clone(),
-                storage: Storage::Local,
-            },
-        );
-
-        var_no
-    }
-
-    pub fn drain(self) -> (Vars, usize) {
-        (self.vars, self.next_id)
-    }
-
-    // In order to create phi nodes, we need to track what vars are set in a certain scope
-    pub fn set_dirty(&mut self, var_no: usize) {
-        for e in &mut self.dirty {
-            if var_no < e.lim {
-                e.set.insert(var_no);
-            }
-        }
-    }
-
-    pub fn new_dirty_tracker(&mut self, lim: usize) {
-        self.dirty.push(DirtyTracker {
-            lim,
-            set: BTreeSet::new(),
-        });
-    }
-
-    pub fn pop_dirty_tracker(&mut self) -> BTreeSet<usize> {
-        self.dirty.pop().unwrap().set
-    }
-}
-
 impl Contract {
     /// Print the entire contract; storage initializers, constructors and functions and their CFGs
     pub fn print_cfg(&self, ns: &Namespace) -> String {
@@ -1828,4 +1675,33 @@ impl Contract {
 
         out
     }
+
+    /// Get the storage slot for a variable, possibly from base contract
+    pub fn get_storage_slot(
+        &self,
+        var_contract_no: usize,
+        var_no: usize,
+        ns: &Namespace,
+    ) -> Expression {
+        if let Some(layout) = self
+            .layout
+            .iter()
+            .find(|l| l.contract_no == var_contract_no && l.var_no == var_no)
+        {
+            Expression::NumberLiteral(pt::Loc::Codegen, ns.storage_type(), layout.slot.clone())
+        } else {
+            panic!("get_storage_slot called on non-storage variable");
+        }
+    }
+}
+
+impl Namespace {
+    /// Type storage
+    pub fn storage_type(&self) -> Type {
+        if self.target == Target::Solana {
+            Type::Uint(32)
+        } else {
+            Type::Uint(256)
+        }
+    }
 }
