@@ -1,10 +1,12 @@
 use super::cfg::{ControlFlowGraph, Instr};
 use super::reaching_definitions;
+use crate::codegen::{Builtin, Expression};
 use crate::parser::pt::Loc;
-use crate::sema::ast::{Builtin, Diagnostic, Expression, Namespace, StringLocation, Type};
+use crate::sema::ast::RetrieveType;
+use crate::sema::ast::{Diagnostic, Namespace, StringLocation, Type};
 use num_bigint::{BigInt, Sign};
 use num_traits::{ToPrimitive, Zero};
-use ripemd160::Ripemd160;
+use ripemd::Ripemd160;
 use sha2::{Digest, Sha256};
 use std::ops::{Add, BitAnd, BitOr, BitXor, Div, Mul, Rem, Shl, Shr, Sub};
 use tiny_keccak::{Hasher, Keccak};
@@ -18,11 +20,9 @@ pub fn constant_folding(cfg: &mut ControlFlowGraph, ns: &mut Namespace) {
         let mut vars = cfg.blocks[block_no].defs.clone();
 
         for instr_no in 0..cfg.blocks[block_no].instr.len() {
-            let cur = reaching_definitions::Def { block_no, instr_no };
-
             match &cfg.blocks[block_no].instr[instr_no] {
                 Instr::Set { loc, res, expr, .. } => {
-                    let (expr, expr_constant) = expression(expr, Some(&vars), &cur, cfg, ns);
+                    let (expr, expr_constant) = expression(expr, Some(&vars), cfg, ns);
 
                     if expr_constant {
                         ns.var_constants.insert(*loc, expr.clone());
@@ -42,7 +42,7 @@ pub fn constant_folding(cfg: &mut ControlFlowGraph, ns: &mut Namespace) {
                 } => {
                     let args = args
                         .iter()
-                        .map(|e| expression(e, Some(&vars), &cur, cfg, ns).0)
+                        .map(|e| expression(e, Some(&vars), cfg, ns).0)
                         .collect();
 
                     cfg.blocks[block_no].instr[instr_no] = Instr::Call {
@@ -55,7 +55,7 @@ pub fn constant_folding(cfg: &mut ControlFlowGraph, ns: &mut Namespace) {
                 Instr::Return { value } => {
                     let value = value
                         .iter()
-                        .map(|e| expression(e, Some(&vars), &cur, cfg, ns).0)
+                        .map(|e| expression(e, Some(&vars), cfg, ns).0)
                         .collect();
 
                     cfg.blocks[block_no].instr[instr_no] = Instr::Return { value };
@@ -65,7 +65,7 @@ pub fn constant_folding(cfg: &mut ControlFlowGraph, ns: &mut Namespace) {
                     true_block,
                     false_block,
                 } => {
-                    let (cond, _) = expression(cond, Some(&vars), &cur, cfg, ns);
+                    let (cond, _) = expression(cond, Some(&vars), cfg, ns);
 
                     if let Expression::BoolLiteral(_, cond) = cond {
                         cfg.blocks[block_no].instr[instr_no] = Instr::Branch {
@@ -80,23 +80,23 @@ pub fn constant_folding(cfg: &mut ControlFlowGraph, ns: &mut Namespace) {
                     }
                 }
                 Instr::Store { dest, pos } => {
-                    let (dest, _) = expression(dest, Some(&vars), &cur, cfg, ns);
+                    let (dest, _) = expression(dest, Some(&vars), cfg, ns);
 
                     cfg.blocks[block_no].instr[instr_no] = Instr::Store { dest, pos: *pos };
                 }
                 Instr::AssertFailure { expr: Some(expr) } => {
-                    let (expr, _) = expression(expr, Some(&vars), &cur, cfg, ns);
+                    let (expr, _) = expression(expr, Some(&vars), cfg, ns);
 
                     cfg.blocks[block_no].instr[instr_no] =
                         Instr::AssertFailure { expr: Some(expr) };
                 }
                 Instr::Print { expr } => {
-                    let (expr, _) = expression(expr, Some(&vars), &cur, cfg, ns);
+                    let (expr, _) = expression(expr, Some(&vars), cfg, ns);
 
                     cfg.blocks[block_no].instr[instr_no] = Instr::Print { expr };
                 }
                 Instr::ClearStorage { ty, storage } => {
-                    let (storage, _) = expression(storage, Some(&vars), &cur, cfg, ns);
+                    let (storage, _) = expression(storage, Some(&vars), cfg, ns);
 
                     cfg.blocks[block_no].instr[instr_no] = Instr::ClearStorage {
                         ty: ty.clone(),
@@ -104,8 +104,8 @@ pub fn constant_folding(cfg: &mut ControlFlowGraph, ns: &mut Namespace) {
                     };
                 }
                 Instr::SetStorage { ty, storage, value } => {
-                    let (storage, _) = expression(storage, Some(&vars), &cur, cfg, ns);
-                    let (value, _) = expression(value, Some(&vars), &cur, cfg, ns);
+                    let (storage, _) = expression(storage, Some(&vars), cfg, ns);
+                    let (value, _) = expression(value, Some(&vars), cfg, ns);
 
                     cfg.blocks[block_no].instr[instr_no] = Instr::SetStorage {
                         ty: ty.clone(),
@@ -113,14 +113,23 @@ pub fn constant_folding(cfg: &mut ControlFlowGraph, ns: &mut Namespace) {
                         value,
                     };
                 }
+                Instr::LoadStorage { ty, storage, res } => {
+                    let (storage, _) = expression(storage, Some(&vars), cfg, ns);
+
+                    cfg.blocks[block_no].instr[instr_no] = Instr::LoadStorage {
+                        ty: ty.clone(),
+                        storage,
+                        res: *res,
+                    };
+                }
                 Instr::SetStorageBytes {
                     storage,
                     value,
                     offset,
                 } => {
-                    let (storage, _) = expression(storage, Some(&vars), &cur, cfg, ns);
-                    let (value, _) = expression(value, Some(&vars), &cur, cfg, ns);
-                    let (offset, _) = expression(offset, Some(&vars), &cur, cfg, ns);
+                    let (storage, _) = expression(storage, Some(&vars), cfg, ns);
+                    let (value, _) = expression(value, Some(&vars), cfg, ns);
+                    let (offset, _) = expression(offset, Some(&vars), cfg, ns);
 
                     cfg.blocks[block_no].instr[instr_no] = Instr::SetStorageBytes {
                         storage,
@@ -130,20 +139,24 @@ pub fn constant_folding(cfg: &mut ControlFlowGraph, ns: &mut Namespace) {
                 }
                 Instr::PushStorage {
                     res,
+                    ty,
                     storage,
                     value,
                 } => {
-                    let (storage, _) = expression(storage, Some(&vars), &cur, cfg, ns);
-                    let (value, _) = expression(value, Some(&vars), &cur, cfg, ns);
+                    let (storage, _) = expression(storage, Some(&vars), cfg, ns);
+                    let value = value
+                        .as_ref()
+                        .map(|expr| expression(expr, Some(&vars), cfg, ns).0);
 
                     cfg.blocks[block_no].instr[instr_no] = Instr::PushStorage {
                         res: *res,
+                        ty: ty.clone(),
                         storage,
                         value,
                     };
                 }
                 Instr::PopStorage { res, ty, storage } => {
-                    let (storage, _) = expression(storage, Some(&vars), &cur, cfg, ns);
+                    let (storage, _) = expression(storage, Some(&vars), cfg, ns);
 
                     cfg.blocks[block_no].instr[instr_no] = Instr::PopStorage {
                         res: *res,
@@ -157,7 +170,7 @@ pub fn constant_folding(cfg: &mut ControlFlowGraph, ns: &mut Namespace) {
                     array,
                     value,
                 } => {
-                    let (value, _) = expression(value, Some(&vars), &cur, cfg, ns);
+                    let (value, _) = expression(value, Some(&vars), cfg, ns);
 
                     cfg.blocks[block_no].instr[instr_no] = Instr::PushMemory {
                         res: *res,
@@ -179,18 +192,18 @@ pub fn constant_folding(cfg: &mut ControlFlowGraph, ns: &mut Namespace) {
                 } => {
                     let args = args
                         .iter()
-                        .map(|e| expression(e, Some(&vars), &cur, cfg, ns).0)
+                        .map(|e| expression(e, Some(&vars), cfg, ns).0)
                         .collect();
                     let value = value
                         .as_ref()
-                        .map(|expr| expression(expr, Some(&vars), &cur, cfg, ns).0);
-                    let gas = expression(gas, Some(&vars), &cur, cfg, ns).0;
+                        .map(|expr| expression(expr, Some(&vars), cfg, ns).0);
+                    let gas = expression(gas, Some(&vars), cfg, ns).0;
                     let salt = salt
                         .as_ref()
-                        .map(|expr| expression(expr, Some(&vars), &cur, cfg, ns).0);
+                        .map(|expr| expression(expr, Some(&vars), cfg, ns).0);
                     let space = space
                         .as_ref()
-                        .map(|expr| expression(expr, Some(&vars), &cur, cfg, ns).0);
+                        .map(|expr| expression(expr, Some(&vars), cfg, ns).0);
 
                     cfg.blocks[block_no].instr[instr_no] = Instr::Constructor {
                         success: *success,
@@ -210,18 +223,23 @@ pub fn constant_folding(cfg: &mut ControlFlowGraph, ns: &mut Namespace) {
                     payload,
                     value,
                     gas,
+                    accounts,
                     callty,
                 } => {
-                    let value = expression(value, Some(&vars), &cur, cfg, ns).0;
-                    let gas = expression(gas, Some(&vars), &cur, cfg, ns).0;
-                    let payload = expression(payload, Some(&vars), &cur, cfg, ns).0;
+                    let value = expression(value, Some(&vars), cfg, ns).0;
+                    let gas = expression(gas, Some(&vars), cfg, ns).0;
+                    let payload = expression(payload, Some(&vars), cfg, ns).0;
                     let address = address
                         .as_ref()
-                        .map(|expr| expression(expr, Some(&vars), &cur, cfg, ns).0);
+                        .map(|expr| expression(expr, Some(&vars), cfg, ns).0);
+                    let accounts = accounts
+                        .as_ref()
+                        .map(|expr| expression(expr, Some(&vars), cfg, ns).0);
 
                     cfg.blocks[block_no].instr[instr_no] = Instr::ExternalCall {
                         success: *success,
                         address,
+                        accounts,
                         payload,
                         value,
                         gas,
@@ -235,7 +253,7 @@ pub fn constant_folding(cfg: &mut ControlFlowGraph, ns: &mut Namespace) {
                     tys,
                     data,
                 } => {
-                    let (data, _) = expression(data, Some(&vars), &cur, cfg, ns);
+                    let (data, _) = expression(data, Some(&vars), cfg, ns);
 
                     cfg.blocks[block_no].instr[instr_no] = Instr::AbiDecode {
                         res: res.clone(),
@@ -246,7 +264,7 @@ pub fn constant_folding(cfg: &mut ControlFlowGraph, ns: &mut Namespace) {
                     }
                 }
                 Instr::SelfDestruct { recipient } => {
-                    let (recipient, _) = expression(recipient, Some(&vars), &cur, cfg, ns);
+                    let (recipient, _) = expression(recipient, Some(&vars), cfg, ns);
 
                     cfg.blocks[block_no].instr[instr_no] = Instr::SelfDestruct { recipient };
                 }
@@ -259,12 +277,12 @@ pub fn constant_folding(cfg: &mut ControlFlowGraph, ns: &mut Namespace) {
                 } => {
                     let data = data
                         .iter()
-                        .map(|e| expression(e, Some(&vars), &cur, cfg, ns).0)
+                        .map(|e| expression(e, Some(&vars), cfg, ns).0)
                         .collect();
 
                     let topics = topics
                         .iter()
-                        .map(|e| expression(e, Some(&vars), &cur, cfg, ns).0)
+                        .map(|e| expression(e, Some(&vars), cfg, ns).0)
                         .collect();
 
                     cfg.blocks[block_no].instr[instr_no] = Instr::EmitEvent {
@@ -293,14 +311,13 @@ pub fn constant_folding(cfg: &mut ControlFlowGraph, ns: &mut Namespace) {
 fn expression(
     expr: &Expression,
     vars: Option<&reaching_definitions::VarDefs>,
-    pos: &reaching_definitions::Def,
     cfg: &ControlFlowGraph,
     ns: &mut Namespace,
 ) -> (Expression, bool) {
     match expr {
         Expression::Add(loc, ty, unchecked, left, right) => {
-            let left = expression(left, vars, pos, cfg, ns);
-            let right = expression(right, vars, pos, cfg, ns);
+            let left = expression(left, vars, cfg, ns);
+            let right = expression(right, vars, cfg, ns);
 
             if let (Expression::NumberLiteral(_, _, left), Expression::NumberLiteral(_, _, right)) =
                 (&left.0, &right.0)
@@ -320,8 +337,8 @@ fn expression(
             }
         }
         Expression::Subtract(loc, ty, unchecked, left, right) => {
-            let left = expression(left, vars, pos, cfg, ns);
-            let right = expression(right, vars, pos, cfg, ns);
+            let left = expression(left, vars, cfg, ns);
+            let right = expression(right, vars, cfg, ns);
 
             if let (Expression::NumberLiteral(_, _, left), Expression::NumberLiteral(_, _, right)) =
                 (&left.0, &right.0)
@@ -341,8 +358,8 @@ fn expression(
             }
         }
         Expression::Multiply(loc, ty, unchecked, left, right) => {
-            let left = expression(left, vars, pos, cfg, ns);
-            let right = expression(right, vars, pos, cfg, ns);
+            let left = expression(left, vars, cfg, ns);
+            let right = expression(right, vars, cfg, ns);
 
             if let (Expression::NumberLiteral(_, _, left), Expression::NumberLiteral(_, _, right)) =
                 (&left.0, &right.0)
@@ -362,8 +379,8 @@ fn expression(
             }
         }
         Expression::BitwiseAnd(loc, ty, left, right) => {
-            let left = expression(left, vars, pos, cfg, ns);
-            let right = expression(right, vars, pos, cfg, ns);
+            let left = expression(left, vars, cfg, ns);
+            let right = expression(right, vars, cfg, ns);
 
             if let (Expression::NumberLiteral(_, _, left), Expression::NumberLiteral(_, _, right)) =
                 (&left.0, &right.0)
@@ -377,8 +394,8 @@ fn expression(
             }
         }
         Expression::BitwiseOr(loc, ty, left, right) => {
-            let left = expression(left, vars, pos, cfg, ns);
-            let right = expression(right, vars, pos, cfg, ns);
+            let left = expression(left, vars, cfg, ns);
+            let right = expression(right, vars, cfg, ns);
 
             if let (Expression::NumberLiteral(_, _, left), Expression::NumberLiteral(_, _, right)) =
                 (&left.0, &right.0)
@@ -392,8 +409,8 @@ fn expression(
             }
         }
         Expression::BitwiseXor(loc, ty, left, right) => {
-            let left = expression(left, vars, pos, cfg, ns);
-            let right = expression(right, vars, pos, cfg, ns);
+            let left = expression(left, vars, cfg, ns);
+            let right = expression(right, vars, cfg, ns);
 
             if let (Expression::NumberLiteral(_, _, left), Expression::NumberLiteral(_, _, right)) =
                 (&left.0, &right.0)
@@ -407,8 +424,8 @@ fn expression(
             }
         }
         Expression::ShiftLeft(loc, ty, left_expr, right_expr) => {
-            let left = expression(left_expr, vars, pos, cfg, ns);
-            let right = expression(right_expr, vars, pos, cfg, ns);
+            let left = expression(left_expr, vars, cfg, ns);
+            let right = expression(right_expr, vars, cfg, ns);
 
             if let (Expression::NumberLiteral(_, _, left), Expression::NumberLiteral(_, _, right)) =
                 (&left.0, &right.0)
@@ -430,8 +447,8 @@ fn expression(
             )
         }
         Expression::ShiftRight(loc, ty, left_expr, right_expr, signed) => {
-            let left = expression(left_expr, vars, pos, cfg, ns);
-            let right = expression(right_expr, vars, pos, cfg, ns);
+            let left = expression(left_expr, vars, cfg, ns);
+            let right = expression(right_expr, vars, cfg, ns);
 
             if let (Expression::NumberLiteral(_, _, left), Expression::NumberLiteral(_, _, right)) =
                 (&left.0, &right.0)
@@ -460,8 +477,8 @@ fn expression(
             )
         }
         Expression::Power(loc, ty, unchecked, left, right) => {
-            let left = expression(left, vars, pos, cfg, ns);
-            let right = expression(right, vars, pos, cfg, ns);
+            let left = expression(left, vars, cfg, ns);
+            let right = expression(right, vars, cfg, ns);
 
             if let (Expression::NumberLiteral(_, _, left), Expression::NumberLiteral(_, _, right)) =
                 (&left.0, &right.0)
@@ -490,8 +507,8 @@ fn expression(
             )
         }
         Expression::Divide(loc, ty, left, right) => {
-            let left = expression(left, vars, pos, cfg, ns);
-            let right = expression(right, vars, pos, cfg, ns);
+            let left = expression(left, vars, cfg, ns);
+            let right = expression(right, vars, cfg, ns);
 
             if let Expression::NumberLiteral(_, _, right) = &right.0 {
                 if right.is_zero() {
@@ -508,8 +525,8 @@ fn expression(
             )
         }
         Expression::Modulo(loc, ty, left, right) => {
-            let left = expression(left, vars, pos, cfg, ns);
-            let right = expression(right, vars, pos, cfg, ns);
+            let left = expression(left, vars, cfg, ns);
+            let right = expression(right, vars, cfg, ns);
 
             if let Expression::NumberLiteral(_, _, right) = &right.0 {
                 if right.is_zero() {
@@ -526,7 +543,7 @@ fn expression(
             )
         }
         Expression::ZeroExt(loc, ty, expr) => {
-            let expr = expression(expr, vars, pos, cfg, ns);
+            let expr = expression(expr, vars, cfg, ns);
             if let Expression::NumberLiteral(_, _, n) = expr.0 {
                 (Expression::NumberLiteral(*loc, ty.clone(), n), true)
             } else {
@@ -537,7 +554,7 @@ fn expression(
             }
         }
         Expression::SignExt(loc, ty, expr) => {
-            let expr = expression(expr, vars, pos, cfg, ns);
+            let expr = expression(expr, vars, cfg, ns);
             if let Expression::NumberLiteral(_, _, n) = expr.0 {
                 (Expression::NumberLiteral(*loc, ty.clone(), n), true)
             } else {
@@ -548,7 +565,7 @@ fn expression(
             }
         }
         Expression::Trunc(loc, ty, expr) => {
-            let expr = expression(expr, vars, pos, cfg, ns);
+            let expr = expression(expr, vars, cfg, ns);
             if let Expression::NumberLiteral(_, _, n) = expr.0 {
                 bigint_to_expression(loc, ty, n)
             } else {
@@ -559,7 +576,7 @@ fn expression(
             }
         }
         Expression::Complement(loc, ty, expr) => {
-            let expr = expression(expr, vars, pos, cfg, ns);
+            let expr = expression(expr, vars, cfg, ns);
             if let Expression::NumberLiteral(_, _, n) = expr.0 {
                 bigint_to_expression(loc, ty, !n)
             } else {
@@ -570,7 +587,7 @@ fn expression(
             }
         }
         Expression::UnaryMinus(loc, ty, expr) => {
-            let expr = expression(expr, vars, pos, cfg, ns);
+            let expr = expression(expr, vars, cfg, ns);
             if let Expression::NumberLiteral(_, _, n) = expr.0 {
                 bigint_to_expression(loc, ty, -n)
             } else {
@@ -581,7 +598,7 @@ fn expression(
             }
         }
         Expression::Variable(loc, ty, var) => {
-            if !matches!(ty, Type::Ref(_) | Type::StorageRef(_, _)) {
+            if !matches!(ty, Type::Ref(_) | Type::StorageRef(..)) {
                 if let Some(vars) = vars {
                     if let Some(defs) = vars.get(var) {
                         // There must be at least one definition, and all should evaluate to the same value
@@ -589,7 +606,7 @@ fn expression(
 
                         for def in defs.keys() {
                             if let Some(expr) = get_definition(def, cfg) {
-                                let expr = expression(expr, None, pos, cfg, ns);
+                                let expr = expression(expr, None, cfg, ns);
 
                                 if expr.1 {
                                     if let Some(last) = &v {
@@ -611,7 +628,7 @@ fn expression(
                         }
 
                         if let Some(expr) = v {
-                            if *loc != Loc(0, 0, 0) {
+                            if *loc != Loc::Codegen {
                                 ns.var_constants.insert(*loc, expr.clone());
                             }
                             return (expr, true);
@@ -623,7 +640,7 @@ fn expression(
             (expr.clone(), false)
         }
         Expression::Builtin(loc, tys, Builtin::Keccak256, args) => {
-            let arg = expression(&args[0], vars, pos, cfg, ns);
+            let arg = expression(&args[0], vars, cfg, ns);
 
             if let Expression::AllocDynamicArray(_, _, _, Some(bs)) = arg.0 {
                 let mut hasher = Keccak::v256();
@@ -643,7 +660,7 @@ fn expression(
             }
         }
         Expression::Builtin(loc, tys, Builtin::Ripemd160, args) => {
-            let arg = expression(&args[0], vars, pos, cfg, ns);
+            let arg = expression(&args[0], vars, cfg, ns);
 
             if let Expression::AllocDynamicArray(_, _, _, Some(bs)) = arg.0 {
                 let mut hasher = Ripemd160::new();
@@ -662,7 +679,7 @@ fn expression(
             }
         }
         Expression::Builtin(loc, tys, Builtin::Blake2_256, args) => {
-            let arg = expression(&args[0], vars, pos, cfg, ns);
+            let arg = expression(&args[0], vars, cfg, ns);
 
             if let Expression::AllocDynamicArray(_, _, _, Some(bs)) = arg.0 {
                 let hash = blake2_rfc::blake2b::blake2b(32, &[], &bs);
@@ -679,7 +696,7 @@ fn expression(
             }
         }
         Expression::Builtin(loc, tys, Builtin::Blake2_128, args) => {
-            let arg = expression(&args[0], vars, pos, cfg, ns);
+            let arg = expression(&args[0], vars, cfg, ns);
 
             if let Expression::AllocDynamicArray(_, _, _, Some(bs)) = arg.0 {
                 let hash = blake2_rfc::blake2b::blake2b(16, &[], &bs);
@@ -696,7 +713,7 @@ fn expression(
             }
         }
         Expression::Builtin(loc, tys, Builtin::Sha256, args) => {
-            let arg = expression(&args[0], vars, pos, cfg, ns);
+            let arg = expression(&args[0], vars, cfg, ns);
 
             if let Expression::AllocDynamicArray(_, _, _, Some(bs)) = arg.0 {
                 let mut hasher = Sha256::new();
@@ -725,7 +742,7 @@ fn expression(
             let args = args
                 .iter()
                 .map(|expr| {
-                    let (expr, _) = expression(expr, vars, pos, cfg, ns);
+                    let (expr, _) = expression(expr, vars, cfg, ns);
 
                     if all_constant {
                         match &expr {
@@ -748,6 +765,9 @@ fn expression(
                                             bs.insert(0, 0);
                                         }
                                     }
+                                    Type::Address(_) => {
+                                        bs.resize(ns.address_length, 0);
+                                    }
                                     _ => unreachable!(),
                                 }
 
@@ -778,7 +798,7 @@ fn expression(
         Expression::StructLiteral(loc, ty, args) => {
             let args = args
                 .iter()
-                .map(|expr| expression(expr, vars, pos, cfg, ns).0)
+                .map(|expr| expression(expr, vars, cfg, ns).0)
                 .collect();
 
             (Expression::StructLiteral(*loc, ty.clone(), args), false)
@@ -786,7 +806,7 @@ fn expression(
         Expression::ArrayLiteral(loc, ty, lengths, args) => {
             let args = args
                 .iter()
-                .map(|expr| expression(expr, vars, pos, cfg, ns).0)
+                .map(|expr| expression(expr, vars, cfg, ns).0)
                 .collect();
 
             (
@@ -797,7 +817,7 @@ fn expression(
         Expression::ConstArrayLiteral(loc, ty, lengths, args) => {
             let args = args
                 .iter()
-                .map(|expr| expression(expr, vars, pos, cfg, ns).0)
+                .map(|expr| expression(expr, vars, cfg, ns).0)
                 .collect();
 
             (
@@ -806,25 +826,17 @@ fn expression(
             )
         }
         Expression::Load(loc, ty, expr) => {
-            let (expr, _) = expression(expr, vars, pos, cfg, ns);
+            let (expr, _) = expression(expr, vars, cfg, ns);
 
             (Expression::Load(*loc, ty.clone(), Box::new(expr)), false)
         }
-        Expression::StorageLoad(loc, ty, expr) => {
-            let (expr, _) = expression(expr, vars, pos, cfg, ns);
-
-            (
-                Expression::StorageLoad(*loc, ty.clone(), Box::new(expr)),
-                false,
-            )
-        }
         Expression::Cast(loc, ty, expr) => {
-            let (expr, _) = expression(expr, vars, pos, cfg, ns);
+            let (expr, _) = expression(expr, vars, cfg, ns);
 
             (Expression::Cast(*loc, ty.clone(), Box::new(expr)), false)
         }
         Expression::BytesCast(loc, from, to, expr) => {
-            let (expr, _) = expression(expr, vars, pos, cfg, ns);
+            let (expr, _) = expression(expr, vars, cfg, ns);
 
             (
                 Expression::BytesCast(*loc, from.clone(), to.clone(), Box::new(expr)),
@@ -832,8 +844,8 @@ fn expression(
             )
         }
         Expression::More(loc, left, right) => {
-            let left = expression(left, vars, pos, cfg, ns);
-            let right = expression(right, vars, pos, cfg, ns);
+            let left = expression(left, vars, cfg, ns);
+            let right = expression(right, vars, cfg, ns);
 
             (
                 Expression::More(*loc, Box::new(left.0), Box::new(right.0)),
@@ -841,8 +853,8 @@ fn expression(
             )
         }
         Expression::Less(loc, left, right) => {
-            let left = expression(left, vars, pos, cfg, ns);
-            let right = expression(right, vars, pos, cfg, ns);
+            let left = expression(left, vars, cfg, ns);
+            let right = expression(right, vars, cfg, ns);
 
             (
                 Expression::Less(*loc, Box::new(left.0), Box::new(right.0)),
@@ -850,8 +862,8 @@ fn expression(
             )
         }
         Expression::MoreEqual(loc, left, right) => {
-            let left = expression(left, vars, pos, cfg, ns);
-            let right = expression(right, vars, pos, cfg, ns);
+            let left = expression(left, vars, cfg, ns);
+            let right = expression(right, vars, cfg, ns);
 
             (
                 Expression::MoreEqual(*loc, Box::new(left.0), Box::new(right.0)),
@@ -859,8 +871,8 @@ fn expression(
             )
         }
         Expression::LessEqual(loc, left, right) => {
-            let left = expression(left, vars, pos, cfg, ns);
-            let right = expression(right, vars, pos, cfg, ns);
+            let left = expression(left, vars, cfg, ns);
+            let right = expression(right, vars, cfg, ns);
 
             (
                 Expression::LessEqual(*loc, Box::new(left.0), Box::new(right.0)),
@@ -868,8 +880,8 @@ fn expression(
             )
         }
         Expression::Equal(loc, left, right) => {
-            let left = expression(left, vars, pos, cfg, ns);
-            let right = expression(right, vars, pos, cfg, ns);
+            let left = expression(left, vars, cfg, ns);
+            let right = expression(right, vars, cfg, ns);
 
             (
                 Expression::Equal(*loc, Box::new(left.0), Box::new(right.0)),
@@ -877,90 +889,50 @@ fn expression(
             )
         }
         Expression::NotEqual(loc, left, right) => {
-            let left = expression(left, vars, pos, cfg, ns);
-            let right = expression(right, vars, pos, cfg, ns);
+            let left = expression(left, vars, cfg, ns);
+            let right = expression(right, vars, cfg, ns);
 
             (
                 Expression::NotEqual(*loc, Box::new(left.0), Box::new(right.0)),
                 false,
             )
         }
-        Expression::Ternary(loc, ty, cond, left, right) => {
-            let cond = expression(cond, vars, pos, cfg, ns);
-            let left = expression(left, vars, pos, cfg, ns);
-            let right = expression(right, vars, pos, cfg, ns);
-
-            (
-                Expression::Ternary(
-                    *loc,
-                    ty.clone(),
-                    Box::new(cond.0),
-                    Box::new(left.0),
-                    Box::new(right.0),
-                ),
-                false,
-            )
-        }
         Expression::Not(loc, expr) => {
-            let expr = expression(expr, vars, pos, cfg, ns);
+            let expr = expression(expr, vars, cfg, ns);
 
             (Expression::Not(*loc, Box::new(expr.0)), expr.1)
         }
-        Expression::Subscript(loc, ty, array, index) => {
-            let array = expression(array, vars, pos, cfg, ns);
-            let index = expression(index, vars, pos, cfg, ns);
+        Expression::Subscript(loc, ty, array_ty, array, index) => {
+            let array = expression(array, vars, cfg, ns);
+            let index = expression(index, vars, cfg, ns);
 
             (
-                Expression::Subscript(*loc, ty.clone(), Box::new(array.0), Box::new(index.0)),
-                false,
-            )
-        }
-        Expression::StructMember(loc, ty, strct, member) => {
-            let strct = expression(strct, vars, pos, cfg, ns);
-
-            (
-                Expression::StructMember(*loc, ty.clone(), Box::new(strct.0), *member),
-                false,
-            )
-        }
-        Expression::DynamicArrayLength(loc, array) => {
-            let array = expression(array, vars, pos, cfg, ns);
-
-            (
-                Expression::DynamicArrayLength(*loc, Box::new(array.0)),
-                false,
-            )
-        }
-        Expression::DynamicArraySubscript(loc, ty, array, index) => {
-            let array = expression(array, vars, pos, cfg, ns);
-            let index = expression(index, vars, pos, cfg, ns);
-
-            (
-                Expression::DynamicArraySubscript(
+                Expression::Subscript(
                     *loc,
                     ty.clone(),
+                    array_ty.clone(),
                     Box::new(array.0),
                     Box::new(index.0),
                 ),
                 false,
             )
         }
-        Expression::StorageBytesSubscript(loc, array, index) => {
-            let array = expression(array, vars, pos, cfg, ns);
-            let index = expression(index, vars, pos, cfg, ns);
+        Expression::StructMember(loc, ty, strct, member) => {
+            let strct = expression(strct, vars, cfg, ns);
 
             (
-                Expression::StorageBytesSubscript(*loc, Box::new(array.0), Box::new(index.0)),
+                Expression::StructMember(*loc, ty.clone(), Box::new(strct.0), *member),
                 false,
             )
         }
+
         Expression::StorageArrayLength {
             loc,
             ty,
             array,
             elem_ty,
         } => {
-            let array = expression(array, vars, pos, cfg, ns);
+            let array = expression(array, vars, cfg, ns);
 
             (
                 Expression::StorageArrayLength {
@@ -979,13 +951,13 @@ fn expression(
                 (Expression::BoolLiteral(*loc, left == right), true)
             } else {
                 let left = if let StringLocation::RunTime(left) = left {
-                    StringLocation::RunTime(Box::new(expression(left, vars, pos, cfg, ns).0))
+                    StringLocation::RunTime(Box::new(expression(left, vars, cfg, ns).0))
                 } else {
                     left.clone()
                 };
 
                 let right = if let StringLocation::RunTime(right) = right {
-                    StringLocation::RunTime(Box::new(expression(right, vars, pos, cfg, ns).0))
+                    StringLocation::RunTime(Box::new(expression(right, vars, cfg, ns).0))
                 } else {
                     right.clone()
                 };
@@ -1005,13 +977,13 @@ fn expression(
                 (Expression::BytesLiteral(*loc, ty.clone(), bs), true)
             } else {
                 let left = if let StringLocation::RunTime(left) = left {
-                    StringLocation::RunTime(Box::new(expression(left, vars, pos, cfg, ns).0))
+                    StringLocation::RunTime(Box::new(expression(left, vars, cfg, ns).0))
                 } else {
                     left.clone()
                 };
 
                 let right = if let StringLocation::RunTime(right) = right {
-                    StringLocation::RunTime(Box::new(expression(right, vars, pos, cfg, ns).0))
+                    StringLocation::RunTime(Box::new(expression(right, vars, cfg, ns).0))
                 } else {
                     right.clone()
                 };
@@ -1025,7 +997,7 @@ fn expression(
         Expression::Builtin(loc, tys, builtin, args) => {
             let args = args
                 .iter()
-                .map(|expr| expression(expr, vars, pos, cfg, ns).0)
+                .map(|expr| expression(expr, vars, cfg, ns).0)
                 .collect();
 
             (
@@ -1039,7 +1011,7 @@ fn expression(
             address,
             function_no,
         } => {
-            let address = expression(address, vars, pos, cfg, ns);
+            let address = expression(address, vars, cfg, ns);
 
             (
                 Expression::ExternalFunction {
@@ -1059,11 +1031,11 @@ fn expression(
         } => {
             let packed = packed
                 .iter()
-                .map(|expr| expression(expr, vars, pos, cfg, ns).0)
+                .map(|expr| expression(expr, vars, cfg, ns).0)
                 .collect();
             let args = args
                 .iter()
-                .map(|expr| expression(expr, vars, pos, cfg, ns).0)
+                .map(|expr| expression(expr, vars, cfg, ns).0)
                 .collect();
 
             (
@@ -1076,16 +1048,17 @@ fn expression(
                 false,
             )
         }
-        Expression::NumberLiteral(_, _, _)
-        | Expression::RationalNumberLiteral(_, _, _)
-        | Expression::BoolLiteral(_, _)
-        | Expression::BytesLiteral(_, _, _)
-        | Expression::CodeLiteral(_, _, _)
-        | Expression::FunctionArg(_, _, _) => (expr.clone(), true),
-        Expression::AllocDynamicArray(_, _, _, _)
+        Expression::NumberLiteral(..)
+        | Expression::RationalNumberLiteral(..)
+        | Expression::BoolLiteral(..)
+        | Expression::BytesLiteral(..)
+        | Expression::CodeLiteral(..)
+        | Expression::FunctionArg(..) => (expr.clone(), true),
+        Expression::AllocDynamicArray(..)
         | Expression::ReturnData(_)
         | Expression::Undefined(_)
         | Expression::FormatString { .. }
+        | Expression::GetRef(..)
         | Expression::InternalFunctionCfg(_) => (expr.clone(), false),
         // nothing else is permitted in cfg
         _ => panic!("expr should not be in cfg: {:?}", expr),
@@ -1114,7 +1087,7 @@ fn bigint_to_expression(loc: &Loc, ty: &Type, n: BigInt) -> (Expression, bool) {
                 n
             }
         }
-        Type::StorageRef(_, _) => n,
+        Type::StorageRef(..) => n,
         _ => unreachable!(),
     };
 
