@@ -1,3 +1,4 @@
+use crate::codegen;
 use crate::codegen::cfg::HashTy;
 use crate::parser::pt;
 use crate::sema::ast;
@@ -10,7 +11,7 @@ use inkwell::context::Context;
 use inkwell::module::Linkage;
 use inkwell::types::IntType;
 use inkwell::values::{
-    BasicMetadataValueEnum, BasicValueEnum, FunctionValue, IntValue, PointerValue,
+    ArrayValue, BasicMetadataValueEnum, BasicValueEnum, FunctionValue, IntValue, PointerValue,
 };
 use inkwell::AddressSpace;
 use inkwell::IntPredicate;
@@ -876,7 +877,7 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
         _function: FunctionValue,
         _ty: &ast::Type,
         _slot: IntValue<'a>,
-        _val: BasicValueEnum<'a>,
+        _val: Option<BasicValueEnum<'a>>,
         _ns: &ast::Namespace,
     ) -> BasicValueEnum<'a> {
         unimplemented!();
@@ -887,8 +888,9 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
         _function: FunctionValue<'a>,
         _ty: &ast::Type,
         _slot: IntValue<'a>,
+        _load: bool,
         _ns: &ast::Namespace,
-    ) -> BasicValueEnum<'a> {
+    ) -> Option<BasicValueEnum<'a>> {
         unimplemented!();
     }
 
@@ -899,12 +901,13 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
         slot: PointerValue,
         dest: PointerValue,
     ) {
-        if dest
-            .get_type()
-            .get_element_type()
-            .into_int_type()
-            .get_bit_width()
-            == 256
+        if !dest.get_type().get_element_type().is_array_type()
+            && dest
+                .get_type()
+                .get_element_type()
+                .into_int_type()
+                .get_bit_width()
+                == 256
         {
             binary.builder.build_call(
                 binary.module.get_function("storageStore").unwrap(),
@@ -1022,6 +1025,55 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
             .into_int_value()
     }
 
+    fn get_storage_address(
+        &self,
+        binary: &Binary<'a>,
+        _function: FunctionValue,
+        slot: PointerValue<'a>,
+        ns: &ast::Namespace,
+    ) -> ArrayValue<'a> {
+        let dest = binary.builder.build_array_alloca(
+            binary.context.i8_type(),
+            binary.context.i32_type().const_int(32, false),
+            "buf",
+        );
+
+        binary.builder.build_call(
+            binary.module.get_function("storageLoad").unwrap(),
+            &[
+                binary
+                    .builder
+                    .build_pointer_cast(
+                        slot,
+                        binary.context.i8_type().ptr_type(AddressSpace::Generic),
+                        "",
+                    )
+                    .into(),
+                binary
+                    .builder
+                    .build_pointer_cast(
+                        dest,
+                        binary.context.i8_type().ptr_type(AddressSpace::Generic),
+                        "",
+                    )
+                    .into(),
+            ],
+            "",
+        );
+
+        binary
+            .builder
+            .build_load(
+                binary.builder.build_pointer_cast(
+                    dest,
+                    binary.address_type(ns).ptr_type(AddressSpace::Generic),
+                    "",
+                ),
+                "loaded_address",
+            )
+            .into_array_value()
+    }
+
     /// ewasm has no keccak256 host function, so call our implementation
     fn keccak256_hash(
         &self,
@@ -1287,11 +1339,6 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
             },
         );
 
-        // address needs its bytes reordered
-        let be_address = binary
-            .builder
-            .build_alloca(binary.address_type(ns), "be_address");
-
         // call create
         let ret = binary
             .builder
@@ -1311,9 +1358,9 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
                     binary
                         .builder
                         .build_pointer_cast(
-                            be_address,
+                            address,
                             binary.context.i8_type().ptr_type(AddressSpace::Generic),
-                            "be_address",
+                            "address",
                         )
                         .into(),
                 ],
@@ -1324,30 +1371,6 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
             .unwrap()
             .into_int_value();
 
-        binary.builder.build_call(
-            binary.module.get_function("__beNtoleN").unwrap(),
-            &[
-                binary
-                    .builder
-                    .build_pointer_cast(
-                        be_address,
-                        binary.context.i8_type().ptr_type(AddressSpace::Generic),
-                        "",
-                    )
-                    .into(),
-                binary
-                    .builder
-                    .build_pointer_cast(
-                        address,
-                        binary.context.i8_type().ptr_type(AddressSpace::Generic),
-                        "",
-                    )
-                    .into(),
-                binary.context.i32_type().const_int(20, false).into(),
-            ],
-            "",
-        );
-
         let is_success = binary.builder.build_int_compare(
             IntPredicate::EQ,
             ret,
@@ -1383,48 +1406,18 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
     fn external_call<'b>(
         &self,
         binary: &Binary<'b>,
-        function: FunctionValue,
+        function: FunctionValue<'b>,
         success: Option<&mut BasicValueEnum<'b>>,
         payload: PointerValue<'b>,
         payload_len: IntValue<'b>,
         address: Option<PointerValue<'b>>,
         gas: IntValue<'b>,
         value: IntValue<'b>,
+        _accounts: Option<(PointerValue<'b>, IntValue<'b>)>,
         callty: ast::CallTy,
         ns: &ast::Namespace,
     ) {
-        // address needs its bytes reordered
-        let be_address = binary
-            .builder
-            .build_alloca(binary.address_type(ns), "be_address");
-
-        binary.builder.build_call(
-            binary.module.get_function("__leNtobeN").unwrap(),
-            &[
-                binary
-                    .builder
-                    .build_pointer_cast(
-                        address.unwrap(),
-                        binary.context.i8_type().ptr_type(AddressSpace::Generic),
-                        "",
-                    )
-                    .into(),
-                binary
-                    .builder
-                    .build_pointer_cast(
-                        be_address,
-                        binary.context.i8_type().ptr_type(AddressSpace::Generic),
-                        "",
-                    )
-                    .into(),
-                binary.context.i32_type().const_int(20, false).into(),
-            ],
-            "",
-        );
-
-        let ret;
-
-        if callty == ast::CallTy::Regular {
+        let ret = if callty == ast::CallTy::Regular {
             // needs value
 
             // value is a u128
@@ -1434,7 +1427,7 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
             binary.builder.build_store(value_ptr, value);
 
             // call create
-            ret = binary
+            binary
                 .builder
                 .build_call(
                     binary.module.get_function("call").unwrap(),
@@ -1443,7 +1436,7 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
                         binary
                             .builder
                             .build_pointer_cast(
-                                be_address,
+                                address.unwrap(),
                                 binary.context.i8_type().ptr_type(AddressSpace::Generic),
                                 "address",
                             )
@@ -1464,9 +1457,9 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
                 .try_as_basic_value()
                 .left()
                 .unwrap()
-                .into_int_value();
+                .into_int_value()
         } else {
-            ret = binary
+            binary
                 .builder
                 .build_call(
                     binary
@@ -1482,7 +1475,7 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
                         binary
                             .builder
                             .build_pointer_cast(
-                                be_address,
+                                address.unwrap(),
                                 binary.context.i8_type().ptr_type(AddressSpace::Generic),
                                 "address",
                             )
@@ -1495,8 +1488,8 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
                 .try_as_basic_value()
                 .left()
                 .unwrap()
-                .into_int_value();
-        }
+                .into_int_value()
+        };
 
         let is_success = binary.builder.build_int_compare(
             IntPredicate::EQ,
@@ -1660,7 +1653,7 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
     }
 
     /// Terminate execution, destroy binary and send remaining funds to addr
-    fn selfdestruct<'b>(&self, binary: &Binary<'b>, addr: IntValue<'b>, ns: &ast::Namespace) {
+    fn selfdestruct<'b>(&self, binary: &Binary<'b>, addr: ArrayValue<'b>, ns: &ast::Namespace) {
         let address = binary
             .builder
             .build_alloca(binary.address_type(ns), "address");
@@ -1679,6 +1672,8 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
                 .into()],
             "terminated",
         );
+
+        binary.builder.build_unreachable();
     }
 
     /// Crypto Hash
@@ -1686,7 +1681,6 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
         &self,
         binary: &Binary<'b>,
         _function: FunctionValue<'b>,
-
         hash: HashTy,
         input: PointerValue<'b>,
         input_len: IntValue<'b>,
@@ -1694,7 +1688,9 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
     ) -> IntValue<'b> {
         let (precompile, hashlen) = match hash {
             HashTy::Keccak256 => (
-                [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 20],
+                [
+                    0u8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 20,
+                ],
                 32,
             ),
             HashTy::Ripemd160 => (
@@ -1722,7 +1718,7 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
             .builder
             .build_store(balance, binary.value_type(ns).const_zero());
 
-        let address = binary.emit_global_string(&format!("precompile_{}", hash), &precompile, true);
+        let address = binary.emit_global_string("precompile", &precompile, true);
 
         binary.builder.build_call(
             binary.module.get_function("call").unwrap(),
@@ -1856,7 +1852,7 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
     fn builtin<'b>(
         &self,
         binary: &Binary<'b>,
-        expr: &ast::Expression,
+        expr: &codegen::Expression,
         vartab: &HashMap<usize, Variable<'b>>,
         function: FunctionValue<'b>,
         ns: &ast::Namespace,
@@ -1872,7 +1868,7 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
             }};
         }
 
-        macro_rules! single_value_stack {
+        macro_rules! single_int_stack {
             ($name:literal, $func:literal, $width:expr) => {{
                 let value = binary
                     .builder
@@ -1895,38 +1891,59 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
             }};
         }
 
+        macro_rules! single_address_stack {
+            ($name:literal, $func:literal) => {{
+                let value = binary.builder.build_alloca(binary.address_type(ns), $name);
+
+                binary.builder.build_call(
+                    binary.module.get_function($func).unwrap(),
+                    &[binary
+                        .builder
+                        .build_pointer_cast(
+                            value,
+                            binary.context.i8_type().ptr_type(AddressSpace::Generic),
+                            "",
+                        )
+                        .into()],
+                    $name,
+                );
+
+                binary.builder.build_load(value, $name)
+            }};
+        }
+
         match expr {
-            ast::Expression::Builtin(_, _, ast::Builtin::BlockNumber, _) => {
+            codegen::Expression::Builtin(_, _, codegen::Builtin::BlockNumber, _) => {
                 straight_call!("block_number", "getBlockNumber")
             }
-            ast::Expression::Builtin(_, _, ast::Builtin::Gasleft, _) => {
+            codegen::Expression::Builtin(_, _, codegen::Builtin::Gasleft, _) => {
                 straight_call!("gas_left", "getGasLeft")
             }
-            ast::Expression::Builtin(_, _, ast::Builtin::GasLimit, _) => {
+            codegen::Expression::Builtin(_, _, codegen::Builtin::GasLimit, _) => {
                 straight_call!("gas_limit", "getBlockGasLimit")
             }
-            ast::Expression::Builtin(_, _, ast::Builtin::Timestamp, _) => {
+            codegen::Expression::Builtin(_, _, codegen::Builtin::Timestamp, _) => {
                 straight_call!("time_stamp", "getBlockTimestamp")
             }
-            ast::Expression::Builtin(_, _, ast::Builtin::BlockDifficulty, _) => {
-                single_value_stack!("block_difficulty", "getBlockDifficulty", 256)
+            codegen::Expression::Builtin(_, _, codegen::Builtin::BlockDifficulty, _) => {
+                single_int_stack!("block_difficulty", "getBlockDifficulty", 256)
             }
-            ast::Expression::Builtin(_, _, ast::Builtin::Origin, _) => {
-                single_value_stack!("origin", "getTxOrigin", ns.address_length as u32 * 8)
+            codegen::Expression::Builtin(_, _, codegen::Builtin::Origin, _) => {
+                single_address_stack!("origin", "getTxOrigin")
             }
-            ast::Expression::Builtin(_, _, ast::Builtin::Sender, _) => {
-                single_value_stack!("caller", "getCaller", ns.address_length as u32 * 8)
+            codegen::Expression::Builtin(_, _, codegen::Builtin::Sender, _) => {
+                single_address_stack!("caller", "getCaller")
             }
-            ast::Expression::Builtin(_, _, ast::Builtin::BlockCoinbase, _) => {
-                single_value_stack!("coinbase", "getBlockCoinbase", ns.address_length as u32 * 8)
+            codegen::Expression::Builtin(_, _, codegen::Builtin::BlockCoinbase, _) => {
+                single_address_stack!("coinbase", "getBlockCoinbase")
             }
-            ast::Expression::Builtin(_, _, ast::Builtin::Gasprice, _) => {
-                single_value_stack!("gas_price", "getTxGasPrice", ns.value_length as u32 * 8)
+            codegen::Expression::Builtin(_, _, codegen::Builtin::Gasprice, _) => {
+                single_int_stack!("gas_price", "getTxGasPrice", ns.value_length as u32 * 8)
             }
-            ast::Expression::Builtin(_, _, ast::Builtin::Value, _) => {
-                single_value_stack!("value", "getCallValue", ns.value_length as u32 * 8)
+            codegen::Expression::Builtin(_, _, codegen::Builtin::Value, _) => {
+                single_int_stack!("value", "getCallValue", ns.value_length as u32 * 8)
             }
-            ast::Expression::Builtin(_, _, ast::Builtin::Calldata, _) => binary
+            codegen::Expression::Builtin(_, _, codegen::Builtin::Calldata, _) => binary
                 .builder
                 .build_call(
                     binary.module.get_function("vector_new").unwrap(),
@@ -1946,7 +1963,7 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
                 .try_as_basic_value()
                 .left()
                 .unwrap(),
-            ast::Expression::Builtin(_, _, ast::Builtin::BlockHash, args) => {
+            codegen::Expression::Builtin(_, _, codegen::Builtin::BlockHash, args) => {
                 let block_number = self.expression(binary, &args[0], vartab, function, ns);
 
                 let value = binary
@@ -1971,7 +1988,7 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
 
                 binary.builder.build_load(value, "block_hash")
             }
-            ast::Expression::Builtin(_, _, ast::Builtin::GetAddress, _) => {
+            codegen::Expression::Builtin(_, _, codegen::Builtin::GetAddress, _) => {
                 let value = binary
                     .builder
                     .build_alloca(binary.address_type(ns), "self_address");
@@ -1991,10 +2008,10 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
 
                 binary.builder.build_load(value, "self_address")
             }
-            ast::Expression::Builtin(_, _, ast::Builtin::Balance, addr) => {
+            codegen::Expression::Builtin(_, _, codegen::Builtin::Balance, addr) => {
                 let addr = self
                     .expression(binary, &addr[0], vartab, function, ns)
-                    .into_int_value();
+                    .into_array_value();
 
                 let address = binary
                     .builder
@@ -2031,7 +2048,7 @@ impl<'a> TargetRuntime<'a> for EwasmTarget {
 
                 binary.builder.build_load(balance, "balance")
             }
-            _ => unimplemented!(),
+            _ => unimplemented!("{:?}", expr),
         }
     }
 }
