@@ -1,13 +1,18 @@
-use super::ast::{Diagnostic, Function, Mutability, Namespace, Parameter, Symbol, Type};
+use super::ast::{
+    BuiltinStruct, Diagnostic, Function, Mutability, Namespace, Parameter, Symbol, Type,
+};
 use super::contracts::is_base;
-use super::tags::resolve_tags;
+use super::tags::{resolve_tags, DocComment};
 use crate::parser::pt;
+use crate::parser::pt::CodeLocation;
+use crate::parser::pt::OptionalCodeLocation;
 use crate::Target;
 
 /// Resolve function declaration in a contract
 pub fn contract_function(
     contract: &pt::ContractDefinition,
     func: &pt::FunctionDefinition,
+    tags: &[DocComment],
     file_no: usize,
     contract_no: usize,
     ns: &mut Namespace,
@@ -30,7 +35,7 @@ pub fn contract_function(
             } else {
                 ns.diagnostics.push(Diagnostic::error(
                     func.name_loc,
-                    "function is missing a name. did you mean ‘fallback() external {…}’ or ‘receive() external {…}’?".to_string(),
+                    "function is missing a name. A function without a name is syntax for 'fallback() external' or 'receive() external' in older versions of the Solidity language, see https://solang.readthedocs.io/en/latest/language.html#fallback-and-receive-function".to_string(),
                 ));
                 return None;
             }
@@ -88,7 +93,7 @@ pub fn contract_function(
     if let Some(loc) = func.return_not_returns {
         ns.diagnostics.push(Diagnostic::error(
             loc,
-            "‘return’ unexpected. Did you mean ‘returns’?".to_string(),
+            "'return' unexpected. Did you mean 'returns'?".to_string(),
         ));
         success = false;
     }
@@ -100,13 +105,21 @@ pub fn contract_function(
 
     for a in &func.attributes {
         match &a {
+            pt::FunctionAttribute::Immutable(loc) => {
+                ns.diagnostics.push(Diagnostic::error(
+                    *loc,
+                    "function cannot be declared 'immutable'".to_string(),
+                ));
+                success = false;
+                continue;
+            }
             pt::FunctionAttribute::Mutability(m) => {
                 if let Some(e) = &mutability {
                     ns.diagnostics.push(Diagnostic::error_with_note(
                         m.loc(),
-                        format!("function redeclared `{}'", m),
+                        format!("function redeclared '{}'", m),
                         e.loc(),
-                        format!("location of previous declaration of `{}'", e),
+                        format!("location of previous declaration of '{}'", e),
                     ));
                     success = false;
                     continue;
@@ -115,7 +128,7 @@ pub fn contract_function(
                 if let pt::Mutability::Constant(loc) = m {
                     ns.diagnostics.push(Diagnostic::warning(
                         *loc,
-                        "‘constant’ is deprecated. Use ‘view’ instead".to_string(),
+                        "'constant' is deprecated. Use 'view' instead".to_string(),
                     ));
 
                     mutability = Some(pt::Mutability::View(*loc));
@@ -126,10 +139,10 @@ pub fn contract_function(
             pt::FunctionAttribute::Visibility(v) => {
                 if let Some(e) = &visibility {
                     ns.diagnostics.push(Diagnostic::error_with_note(
-                        v.loc(),
-                        format!("function redeclared `{}'", v),
-                        e.loc(),
-                        format!("location of previous declaration of `{}'", e),
+                        v.loc().unwrap(),
+                        format!("function redeclared '{}'", v),
+                        e.loc().unwrap(),
+                        format!("location of previous declaration of '{}'", e),
                     ));
                     success = false;
                     continue;
@@ -141,9 +154,9 @@ pub fn contract_function(
                 if let Some(prev_loc) = &is_virtual {
                     ns.diagnostics.push(Diagnostic::error_with_note(
                         *loc,
-                        "function redeclared ‘virtual’".to_string(),
+                        "function redeclared 'virtual'".to_string(),
                         *prev_loc,
-                        "location of previous declaration of ‘virtual’".to_string(),
+                        "location of previous declaration of 'virtual'".to_string(),
                     ));
                     success = false;
                     continue;
@@ -155,9 +168,9 @@ pub fn contract_function(
                 if let Some((prev_loc, _)) = &is_override {
                     ns.diagnostics.push(Diagnostic::error_with_note(
                         *loc,
-                        "function redeclared ‘override’".to_string(),
+                        "function redeclared 'override'".to_string(),
                         *prev_loc,
-                        "location of previous declaration of ‘override’".to_string(),
+                        "location of previous declaration of 'override'".to_string(),
                     ));
                     success = false;
                     continue;
@@ -171,13 +184,13 @@ pub fn contract_function(
                             if list.contains(&no) {
                                 ns.diagnostics.push(Diagnostic::error(
                                     name.loc,
-                                    format!("function duplicate override ‘{}’", name.name),
+                                    format!("function duplicate override '{}'", name.name),
                                 ));
                             } else if !is_base(no, contract_no, ns) {
                                 ns.diagnostics.push(Diagnostic::error(
                                     name.loc,
                                     format!(
-                                        "override ‘{}’ is not a base contract of ‘{}’",
+                                        "override '{}' is not a base contract of '{}'",
                                         name.name, ns.contracts[contract_no].name
                                     ),
                                 ));
@@ -188,7 +201,7 @@ pub fn contract_function(
                         None => {
                             ns.diagnostics.push(Diagnostic::error(
                                 name.loc,
-                                format!("contract ‘{}’ in override list not found", name.name),
+                                format!("contract '{}' in override list not found", name.name),
                             ));
                         }
                     }
@@ -217,15 +230,15 @@ pub fn contract_function(
         Some(v) => {
             if func.ty == pt::FunctionTy::Modifier {
                 ns.diagnostics.push(Diagnostic::error(
-                    v.loc(),
-                    format!("‘{}’: modifiers can not have visibility", v),
+                    v.loc().unwrap(),
+                    format!("'{}': modifiers can not have visibility", v),
                 ));
 
                 pt::Visibility::Internal(v.loc())
             } else if func.ty == pt::FunctionTy::Constructor {
                 ns.diagnostics.push(Diagnostic::warning(
-                    v.loc(),
-                    format!("‘{}’: visibility for constructors is ignored", v),
+                    v.loc().unwrap(),
+                    format!("'{}': visibility for constructors is ignored", v),
                 ));
 
                 pt::Visibility::Public(v.loc())
@@ -235,8 +248,8 @@ pub fn contract_function(
         }
         None => {
             match func.ty {
-                pt::FunctionTy::Constructor => pt::Visibility::Public(pt::Loc(0, 0, 0)),
-                pt::FunctionTy::Modifier => pt::Visibility::Internal(pt::Loc(0, 0, 0)),
+                pt::FunctionTy::Constructor => pt::Visibility::Public(None),
+                pt::FunctionTy::Modifier => pt::Visibility::Internal(None),
                 _ => {
                     ns.diagnostics.push(Diagnostic::error(
                         func.loc,
@@ -245,7 +258,7 @@ pub fn contract_function(
 
                     success = false;
                     // continue processing while assuming it's a public
-                    pt::Visibility::Public(pt::Loc(0, 0, 0))
+                    pt::Visibility::Public(None)
                 }
             }
         }
@@ -321,7 +334,7 @@ pub fn contract_function(
         } else {
             ns.diagnostics.push(Diagnostic::error(
                 func.loc,
-                "functions must be declared ‘external’ in an interface".to_string(),
+                "functions must be declared 'external' in an interface".to_string(),
             ));
             success = false;
         }
@@ -354,7 +367,7 @@ pub fn contract_function(
     } else if func.ty == pt::FunctionTy::Constructor && is_virtual.is_some() {
         ns.diagnostics.push(Diagnostic::error(
             func.loc,
-            "constructors cannot be declared ‘virtual’".to_string(),
+            "constructors cannot be declared 'virtual'".to_string(),
         ));
     }
 
@@ -384,7 +397,7 @@ pub fn contract_function(
     if !is_virtual && func.body.is_none() {
         ns.diagnostics.push(Diagnostic::error(
             func.loc,
-            "function with no body must be marked ‘virtual’".to_string(),
+            "function with no body missing 'virtual'. This was permitted in older versions of the Solidity language, please update.".to_string(),
         ));
         success = false;
     }
@@ -393,7 +406,7 @@ pub fn contract_function(
         if is_virtual {
             ns.diagnostics.push(Diagnostic::error(
                 func.loc,
-                "function marked ‘virtual’ cannot also be ‘private’".to_string(),
+                "function marked 'virtual' cannot also be 'private'".to_string(),
             ));
             success = false;
         }
@@ -408,16 +421,16 @@ pub fn contract_function(
         None => "".to_owned(),
     };
 
-    let bases: Vec<&str> = contract
+    let bases: Vec<String> = contract
         .base
         .iter()
-        .map(|base| -> &str { &base.name.name })
+        .map(|base| format!("{}", base.name))
         .collect();
 
     let doc = resolve_tags(
-        func.loc.0,
+        func.loc.file_no(),
         "function",
-        &func.doc,
+        tags,
         Some(&params),
         Some(&returns),
         Some(&bases),
@@ -471,7 +484,7 @@ pub fn contract_function(
 
                 ns.diagnostics.push(Diagnostic::error_with_note(
                     func.loc,
-                    "all constructors should be defined ‘payable’ or not".to_string(),
+                    "all constructors should be defined 'payable' or not".to_string(),
                     prev_loc,
                     "location of previous definition".to_string(),
                 ));
@@ -612,10 +625,11 @@ pub fn contract_function(
     }
 }
 
-/// Resolve function declaration outside a contract
+/// Resolve free function
 pub fn function(
     func: &pt::FunctionDefinition,
     file_no: usize,
+    tags: &[DocComment],
     ns: &mut Namespace,
 ) -> Option<usize> {
     let mut success = true;
@@ -624,13 +638,21 @@ pub fn function(
 
     for a in &func.attributes {
         match &a {
+            pt::FunctionAttribute::Immutable(loc) => {
+                ns.diagnostics.push(Diagnostic::error(
+                    *loc,
+                    "function cannot be declared 'immutable'".to_string(),
+                ));
+                success = false;
+                continue;
+            }
             pt::FunctionAttribute::Mutability(m) => {
                 if let Some(e) = &mutability {
                     ns.diagnostics.push(Diagnostic::error_with_note(
                         m.loc(),
-                        format!("function redeclared `{}'", m),
+                        format!("function redeclared '{}'", m),
                         e.loc(),
-                        format!("location of previous declaration of `{}'", e),
+                        format!("location of previous declaration of '{}'", e),
                     ));
                     success = false;
                     continue;
@@ -639,7 +661,7 @@ pub fn function(
                 if let pt::Mutability::Constant(loc) = m {
                     ns.diagnostics.push(Diagnostic::warning(
                         *loc,
-                        "‘constant’ is deprecated. Use ‘view’ instead".to_string(),
+                        "'constant' is deprecated. Use 'view' instead".to_string(),
                     ));
 
                     mutability = Some(pt::Mutability::View(*loc));
@@ -649,9 +671,9 @@ pub fn function(
             }
             pt::FunctionAttribute::Visibility(v) => {
                 ns.diagnostics.push(Diagnostic::error(
-                    v.loc(),
+                    v.loc().unwrap(),
                     format!(
-                        "‘{}’: only functions in contracts can have a visibility specifier",
+                        "'{}': only functions in contracts can have a visibility specifier",
                         v
                     ),
                 ));
@@ -720,9 +742,9 @@ pub fn function(
     };
 
     let doc = resolve_tags(
-        func.loc.0,
+        func.loc.file_no(),
         "function",
-        &func.doc,
+        tags,
         Some(&params),
         Some(&returns),
         None,
@@ -736,7 +758,7 @@ pub fn function(
         doc,
         func.ty,
         mutability,
-        pt::Visibility::Internal(func.loc),
+        pt::Visibility::Internal(None),
         params,
         returns,
         ns,
@@ -799,19 +821,30 @@ pub fn resolve_params(
 
         match ns.resolve_type(file_no, contract_no, false, &p.ty, diagnostics) {
             Ok(ty) => {
-                if !is_internal && ty.contains_internal_function(ns) {
-                    diagnostics.push(Diagnostic::error(
+                if !is_internal {
+                    if ty.contains_internal_function(ns) {
+                        diagnostics.push(Diagnostic::error(
                         p.ty.loc(),
                         "parameter of type ‘function internal’ not allowed public or external functions".to_string(),
                     ));
-                    success = false;
+                        success = false;
+                    }
+
+                    if let Some(ty) = ty.contains_builtins(ns, BuiltinStruct::AccountInfo) {
+                        let message = format!(
+                            "parameter of type '{}' not alowed in public or external functions",
+                            ty.to_string(ns)
+                        );
+                        ns.diagnostics.push(Diagnostic::error(p.ty.loc(), message));
+                        success = false
+                    }
                 }
 
                 let ty = if !ty.can_have_data_location() {
                     if let Some(storage) = &p.storage {
                         diagnostics.push(Diagnostic::error(
-                            *storage.loc(),
-                                format!("data location ‘{}’ can only be specified for array, struct or mapping",
+                            storage.loc(),
+                                format!("data location '{}' can only be specified for array, struct or mapping",
                                 storage)
                             ));
                         success = false;
@@ -822,20 +855,20 @@ pub fn resolve_params(
                     if !is_internal {
                         diagnostics.push(Diagnostic::error(
                             loc,
-                            "parameter of type ‘storage’ not allowed public or external functions"
+                            "parameter of type 'storage' not allowed public or external functions"
                                 .to_string(),
                         ));
                         success = false;
                     }
 
-                    ty_loc.2 = loc.2;
+                    ty_loc.use_end_from(&loc);
 
                     Type::StorageRef(false, Box::new(ty))
                 } else {
                     if ty.contains_mapping(ns) {
                         diagnostics.push(Diagnostic::error(
                             p.ty.loc(),
-                            "parameter with mapping type must be of type ‘storage’".to_string(),
+                            "parameter with mapping type must be of type 'storage'".to_string(),
                         ));
                         success = false;
                     }
@@ -853,14 +886,11 @@ pub fn resolve_params(
 
                 params.push(Parameter {
                     loc: *loc,
-                    name: p
-                        .name
-                        .as_ref()
-                        .map_or("".to_string(), |id| id.name.to_string()),
-                    name_loc: p.name.as_ref().map(|id| id.loc),
+                    id: p.name.clone(),
                     ty,
-                    ty_loc,
+                    ty_loc: Some(ty_loc),
                     indexed: false,
+                    readonly: false,
                 });
             }
             Err(()) => success = false,
@@ -896,20 +926,30 @@ pub fn resolve_returns(
 
         match ns.resolve_type(file_no, contract_no, false, &r.ty, diagnostics) {
             Ok(ty) => {
-                if !is_internal && ty.contains_internal_function(ns) {
-                    diagnostics.push(Diagnostic::error(
+                if !is_internal {
+                    if ty.contains_internal_function(ns) {
+                        diagnostics.push(Diagnostic::error(
                         r.ty.loc(),
-                        "return type ‘function internal’ not allowed public or external functions"
+                        "return type ‘function internal’ not allowed in public or external functions"
                             .to_string(),
                     ));
-                    success = false;
-                }
+                        success = false;
+                    }
 
+                    if let Some(ty) = ty.contains_builtins(ns, BuiltinStruct::AccountInfo) {
+                        let message = format!(
+                            "return type '{}' not allowed in public or external functions",
+                            ty.to_string(ns)
+                        );
+                        ns.diagnostics.push(Diagnostic::error(r.ty.loc(), message));
+                        success = false
+                    }
+                }
                 let ty = if !ty.can_have_data_location() {
                     if let Some(storage) = &r.storage {
                         diagnostics.push(Diagnostic::error(
-                            *storage.loc(),
-                                format!("data location ‘{}’ can only be specified for array, struct or mapping",
+                            storage.loc(),
+                                format!("data location '{}' can only be specified for array, struct or mapping",
                                 storage)
                             ));
                         success = false;
@@ -922,13 +962,13 @@ pub fn resolve_returns(
                             if !is_internal {
                                 diagnostics.push(Diagnostic::error(
                                     loc,
-                                    "return type of type ‘storage’ not allowed public or external functions"
+                                    "return type of type 'storage' not allowed public or external functions"
                                         .to_string(),
                                 ));
                                 success = false;
                             }
 
-                            ty_loc.2 = loc.2;
+                            ty_loc.use_end_from(&loc);
 
                             Type::StorageRef(false, Box::new(ty))
                         }
@@ -936,7 +976,7 @@ pub fn resolve_returns(
                             if ty.contains_mapping(ns) {
                                 diagnostics.push(Diagnostic::error(
                                     r.ty.loc(),
-                                    "return type containing mapping must be of type ‘storage’"
+                                    "return type containing mapping must be of type 'storage'"
                                         .to_string(),
                                 ));
                                 success = false;
@@ -957,14 +997,11 @@ pub fn resolve_returns(
 
                 resolved_returns.push(Parameter {
                     loc: *loc,
-                    name: r
-                        .name
-                        .as_ref()
-                        .map_or("".to_string(), |id| id.name.to_string()),
-                    name_loc: r.name.as_ref().map(|id| id.loc),
+                    id: r.name.clone(),
                     ty,
-                    ty_loc,
+                    ty_loc: Some(ty_loc),
                     indexed: false,
+                    readonly: false,
                 });
             }
             Err(()) => success = false,
@@ -982,35 +1019,35 @@ fn signatures() {
 
     ns.contracts.push(ast::Contract::new(
         "bar",
-        pt::ContractTy::Contract(pt::Loc(0, 0, 0)),
+        pt::ContractTy::Contract(pt::Loc::Implicit),
         Vec::new(),
-        pt::Loc(0, 0, 0),
+        pt::Loc::Implicit,
     ));
 
     let fdecl = Function::new(
-        pt::Loc(0, 0, 0),
+        pt::Loc::Implicit,
         "foo".to_owned(),
         None,
         vec![],
         pt::FunctionTy::Function,
         None,
-        pt::Visibility::Public(pt::Loc(0, 0, 0)),
+        pt::Visibility::Public(None),
         vec![
             Parameter {
-                loc: pt::Loc(0, 0, 0),
-                name: "".to_string(),
-                name_loc: None,
+                loc: pt::Loc::Implicit,
+                id: None,
                 ty: Type::Uint(8),
-                ty_loc: pt::Loc(0, 0, 0),
+                ty_loc: None,
                 indexed: false,
+                readonly: false,
             },
             Parameter {
-                loc: pt::Loc(0, 0, 0),
-                name: "".to_string(),
-                name_loc: None,
+                loc: pt::Loc::Implicit,
+                id: None,
                 ty: Type::Address(false),
-                ty_loc: pt::Loc(0, 0, 0),
+                ty_loc: None,
                 indexed: false,
+                readonly: false,
             },
         ],
         Vec::new(),
