@@ -1,24 +1,23 @@
-use ethabi::{decode, RawLog, Token};
+use ethabi::{decode, ethereum_types, RawLog, Token};
 use num_derive::FromPrimitive;
 use num_traits::FromPrimitive;
 use rand::Rng;
-use ripemd160::Ripemd160;
+use ripemd::Ripemd160;
 use sha2::{Digest, Sha256};
 use std::collections::HashMap;
+use std::ffi::OsStr;
 use std::fmt;
 use tiny_keccak::{Hasher, Keccak};
 use wasmi::memory_units::Pages;
 use wasmi::*;
 
-use solang::file_resolver::FileResolver;
-use solang::sema::{ast, diagnostics};
-use solang::{compile, Target};
+use solang::{compile, file_resolver::FileResolver, Target};
 
 mod ewasm_tests;
 
 type Address = [u8; 20];
 
-fn address_new() -> Address {
+pub fn address_new() -> Address {
     let mut rng = rand::thread_rng();
 
     let mut a = [0u8; 20];
@@ -54,6 +53,7 @@ struct TestRuntime {
     abi: ethabi::Contract,
     contracts: Vec<Vec<u8>>,
     value: u128,
+    caller: Address,
     accounts: HashMap<Address, (Vec<u8>, u128)>,
     store: HashMap<(Address, [u8; 32]), [u8; 32]>,
     vm: VirtualMachine,
@@ -82,6 +82,7 @@ pub enum Extern {
     returnDataCopy,
     getReturnDataSize,
     getCallValue,
+    getCaller,
     getAddress,
     getExternalBalance,
     selfDestruct,
@@ -465,6 +466,18 @@ impl Externals for TestRuntime {
 
                 Ok(None)
             }
+            Some(Extern::getCaller) => {
+                let address_ptr: u32 = args.nth_checked(0)?;
+
+                println!("getCaller: {}", hex::encode(&self.caller));
+
+                self.vm
+                    .memory
+                    .set(address_ptr, &self.caller[..])
+                    .expect("set address");
+
+                Ok(None)
+            }
             Some(Extern::getExternalBalance) => {
                 let address_ptr: u32 = args.nth_checked(0)?;
                 let balance_ptr: u32 = args.nth_checked(1)?;
@@ -573,6 +586,7 @@ impl ModuleImportResolver for TestRuntime {
             "returnDataCopy" => Extern::returnDataCopy,
             "getReturnDataSize" => Extern::getReturnDataSize,
             "getCallValue" => Extern::getCallValue,
+            "getCaller" => Extern::getCaller,
             "getAddress" => Extern::getAddress,
             "getExternalBalance" => Extern::getExternalBalance,
             "selfDestruct" => Extern::selfDestruct,
@@ -802,14 +816,14 @@ fn build_solidity(src: &str) -> TestRuntime {
     cache.set_file_contents("test.sol", src.to_string());
 
     let (res, ns) = compile(
-        "test.sol",
+        OsStr::new("test.sol"),
         &mut cache,
         inkwell::OptimizationLevel::Default,
         Target::Ewasm,
         false,
     );
 
-    diagnostics::print_messages(&cache, &ns, false);
+    ns.print_diagnostics_in_plain(&cache, false);
 
     for v in &res {
         println!("contract size:{}", v.0.len());
@@ -828,6 +842,7 @@ fn build_solidity(src: &str) -> TestRuntime {
         abi: ethabi::Contract::load(abi.as_bytes()).unwrap(),
         contracts: res.into_iter().map(|v| v.0).collect(),
         events: Vec::new(),
+        caller: address_new(),
     }
 }
 
@@ -854,31 +869,6 @@ fn simple_solidiy_compile_and_run() {
     );
 }
 
-pub fn parse_and_resolve(src: &'static str, target: Target) -> ast::Namespace {
-    let mut cache = FileResolver::new();
-
-    cache.set_file_contents("test.sol", src.to_string());
-
-    solang::parse_and_resolve("test.sol", &mut cache, target)
-}
-
-pub fn first_error(errors: Vec<ast::Diagnostic>) -> String {
-    match errors.iter().find(|m| m.level == ast::Level::Error) {
-        Some(m) => m.message.to_owned(),
-        None => panic!("no errors found"),
-    }
-}
-
-pub fn no_errors(errors: Vec<ast::Diagnostic>) {
-    assert!(
-        errors
-            .iter()
-            .filter(|m| m.level == ast::Level::Error)
-            .count()
-            == 0
-    );
-}
-
 #[test]
 fn simple_loops() {
     let mut runtime = build_solidity(
@@ -1516,6 +1506,9 @@ fn storage_structs() {
     // verified on remix
     let mut runtime = build_solidity(
         r##"
+        /**
+         * This is a doccomment
+         */
         pragma solidity 0;
         pragma experimental ABIEncoderV2;
 
@@ -2291,7 +2284,7 @@ fn selfdestruct() {
             }
 
             function step2() public {
-                o.goaway(payable(address(this)));
+                bool foo = o.goaway(payable(address(this)));
             }
         }"##,
     );
