@@ -1,5 +1,5 @@
 use crate::build_solidity;
-use ethereum_types::Address;
+use ethabi::ethereum_types::{Address, U256};
 use num_bigint::{BigInt, BigUint};
 use rand::Rng;
 use std::ops::Add;
@@ -152,15 +152,9 @@ fn test_enum() {
 
     let returns = vm.function("return_enum", &[]);
 
-    assert_eq!(
-        returns,
-        vec![ethabi::Token::Uint(ethereum_types::U256::from(9))]
-    );
+    assert_eq!(returns, vec![ethabi::Token::Uint(U256::from(9))]);
 
-    vm.function(
-        "enum_arg",
-        &[ethabi::Token::Uint(ethereum_types::U256::from(6))],
-    );
+    vm.function("enum_arg", &[ethabi::Token::Uint(U256::from(6))]);
 }
 
 #[test]
@@ -282,7 +276,7 @@ fn bytes() {
                 "shift_left",
                 &[
                     ethabi::Token::FixedBytes(a.to_vec()),
-                    ethabi::Token::Uint(ethereum_types::U256::from(r)),
+                    ethabi::Token::Uint(U256::from(r)),
                 ],
             );
 
@@ -302,7 +296,7 @@ fn bytes() {
                 "shift_right",
                 &[
                     ethabi::Token::FixedBytes(a.to_vec()),
-                    ethabi::Token::Uint(ethereum_types::U256::from(r)),
+                    ethabi::Token::Uint(U256::from(r)),
                 ],
             );
 
@@ -384,8 +378,8 @@ fn uint() {
             rng.fill(&mut a[..]);
             rng.fill(&mut b[..]);
 
-            let mut a = ethereum_types::U256::from_big_endian(&a);
-            let mut b = ethereum_types::U256::from_big_endian(&b);
+            let mut a = U256::from_big_endian(&a);
+            let mut b = U256::from_big_endian(&b);
 
             rng.fill(&mut a.0[..]);
             rng.fill(&mut b.0[..]);
@@ -425,7 +419,7 @@ fn uint() {
 
             assert_eq!(pow, vec![ethabi::Token::Uint(res)]);
 
-            if b != ethereum_types::U256::zero() {
+            if b != U256::zero() {
                 let div = vm.function("div", &[ethabi::Token::Uint(a), ethabi::Token::Uint(b)]);
 
                 let mut res = a.div(b);
@@ -446,7 +440,7 @@ fn uint() {
 
             let or = vm.function("or", &[ethabi::Token::Uint(a), ethabi::Token::Uint(b)]);
 
-            let mut res = ethereum_types::U256([
+            let mut res = U256([
                 a.0[0] | b.0[0],
                 a.0[1] | b.0[1],
                 a.0[2] | b.0[2],
@@ -459,7 +453,7 @@ fn uint() {
 
             let and = vm.function("and", &[ethabi::Token::Uint(a), ethabi::Token::Uint(b)]);
 
-            let mut res = ethereum_types::U256([
+            let mut res = U256([
                 a.0[0] & b.0[0],
                 a.0[1] & b.0[1],
                 a.0[2] & b.0[2],
@@ -472,7 +466,7 @@ fn uint() {
 
             let xor = vm.function("xor", &[ethabi::Token::Uint(a), ethabi::Token::Uint(b)]);
 
-            let mut res = ethereum_types::U256([
+            let mut res = U256([
                 a.0[0] ^ b.0[0],
                 a.0[1] ^ b.0[1],
                 a.0[2] ^ b.0[2],
@@ -487,10 +481,7 @@ fn uint() {
 
             let shl = vm.function(
                 "shift_left",
-                &[
-                    ethabi::Token::Uint(a),
-                    ethabi::Token::Uint(ethereum_types::U256::from(r)),
-                ],
+                &[ethabi::Token::Uint(a), ethabi::Token::Uint(U256::from(r))],
             );
 
             let mut res = a.shl(r);
@@ -501,10 +492,7 @@ fn uint() {
 
             let shr = vm.function(
                 "shift_right",
-                &[
-                    ethabi::Token::Uint(a),
-                    ethabi::Token::Uint(ethereum_types::U256::from(r)),
-                ],
+                &[ethabi::Token::Uint(a), ethabi::Token::Uint(U256::from(r))],
             );
 
             let mut res = a.shr(r);
@@ -516,7 +504,7 @@ fn uint() {
     }
 }
 
-fn truncate_uint(n: &mut ethereum_types::U256, width: usize) {
+fn truncate_uint(n: &mut U256, width: usize) {
     let mut bits = 256 - width;
 
     let mut offset = 3;
@@ -596,8 +584,8 @@ fn int() {
             rng.fill(&mut a_bs[..]);
             rng.fill(&mut b_bs[..]);
 
-            let mut a = ethereum_types::U256::from_big_endian(&a_bs);
-            let mut b = ethereum_types::U256::from_big_endian(&b_bs);
+            let mut a = U256::from_big_endian(&a_bs);
+            let mut b = U256::from_big_endian(&b_bs);
 
             truncate_int(&mut a, width);
             truncate_int(&mut b, width);
@@ -625,7 +613,7 @@ fn int() {
 
             assert_eq!(mul, vec![ethabi::Token::Int(res)]);
 
-            if b != ethereum_types::U256::zero() {
+            if b != U256::zero() {
                 let div = vm.function("div", &[ethabi::Token::Int(a), ethabi::Token::Int(b)]);
 
                 let res = bigint_to_eth(&big_a.clone().div(&big_b), width);
@@ -643,7 +631,7 @@ fn int() {
 
             let or = vm.function("or", &[ethabi::Token::Int(a), ethabi::Token::Int(b)]);
 
-            let mut res = ethereum_types::U256([
+            let mut res = U256([
                 a.0[0] | b.0[0],
                 a.0[1] | b.0[1],
                 a.0[2] | b.0[2],
@@ -656,7 +644,7 @@ fn int() {
 
             let and = vm.function("and", &[ethabi::Token::Int(a), ethabi::Token::Int(b)]);
 
-            let mut res = ethereum_types::U256([
+            let mut res = U256([
                 a.0[0] & b.0[0],
                 a.0[1] & b.0[1],
                 a.0[2] & b.0[2],
@@ -669,7 +657,7 @@ fn int() {
 
             let xor = vm.function("xor", &[ethabi::Token::Int(a), ethabi::Token::Int(b)]);
 
-            let mut res = ethereum_types::U256([
+            let mut res = U256([
                 a.0[0] ^ b.0[0],
                 a.0[1] ^ b.0[1],
                 a.0[2] ^ b.0[2],
@@ -684,10 +672,7 @@ fn int() {
 
             let shl = vm.function(
                 "shift_left",
-                &[
-                    ethabi::Token::Int(a),
-                    ethabi::Token::Uint(ethereum_types::U256::from(r)),
-                ],
+                &[ethabi::Token::Int(a), ethabi::Token::Uint(U256::from(r))],
             );
 
             let mut res = a.shl(r);
@@ -698,10 +683,7 @@ fn int() {
 
             let shr = vm.function(
                 "shift_right",
-                &[
-                    ethabi::Token::Int(a),
-                    ethabi::Token::Uint(ethereum_types::U256::from(r)),
-                ],
+                &[ethabi::Token::Int(a), ethabi::Token::Uint(U256::from(r))],
             );
 
             let res = bigint_to_eth(&big_a.clone().shr(r), width);
@@ -711,9 +693,8 @@ fn int() {
     }
 }
 
-fn truncate_int(n: &mut ethereum_types::U256, width: usize) {
-    let sign =
-        n.bitand(ethereum_types::U256::from(1) << (width - 1)) != ethereum_types::U256::zero();
+fn truncate_int(n: &mut U256, width: usize) {
+    let sign = n.bitand(U256::from(1) << (width - 1)) != U256::zero();
 
     let mut bits = 256 - width;
 
@@ -735,7 +716,7 @@ fn truncate_int(n: &mut ethereum_types::U256, width: usize) {
     }
 }
 
-fn bigint_to_eth(v: &BigInt, width: usize) -> ethereum_types::U256 {
+fn bigint_to_eth(v: &BigInt, width: usize) -> U256 {
     let mut buf = v.to_signed_bytes_be();
     let width = width / 8;
 
@@ -749,10 +730,10 @@ fn bigint_to_eth(v: &BigInt, width: usize) -> ethereum_types::U256 {
         buf.insert(0, sign);
     }
 
-    ethereum_types::U256::from_big_endian(&buf)
+    U256::from_big_endian(&buf)
 }
 
-fn eth_to_bigint(v: &ethereum_types::U256, width: usize) -> BigInt {
+fn eth_to_bigint(v: &U256, width: usize) -> BigInt {
     let mut buf = Vec::new();
 
     buf.resize(32, 0);
