@@ -2,7 +2,7 @@ mod solana_helpers;
 
 use base58::{FromBase58, ToBase58};
 use byteorder::{ByteOrder, LittleEndian, WriteBytesExt};
-use ethabi::{RawLog, Token};
+use ethabi::{ethereum_types::H256, RawLog, Token};
 use libc::c_char;
 use rand::Rng;
 use serde::{Deserialize, Serialize};
@@ -10,11 +10,12 @@ use sha2::{Digest, Sha256};
 use solana_helpers::allocator_bump::Allocator;
 use solana_rbpf::{
     ebpf,
+    elf::Executable,
     error::EbpfError,
     memory_region::{AccessType, MemoryMapping},
     question_mark,
     user_error::UserError,
-    vm::{Config, EbpfVm, Executable, SyscallObject, SyscallRegistry, TestInstructionMeter},
+    vm::{Config, EbpfVm, SyscallObject, SyscallRegistry, TestInstructionMeter},
 };
 use solang::{
     abi::generate_abi,
@@ -22,16 +23,18 @@ use solang::{
     compile_many,
     emit::Generate,
     file_resolver::FileResolver,
-    sema::{ast, diagnostics},
     Target,
 };
-use std::alloc::Layout;
-use std::cell::RefCell;
-use std::collections::HashMap;
-use std::convert::TryInto;
-use std::io::Write;
-use std::mem::{align_of, size_of};
-use std::rc::Rc;
+use std::{
+    alloc::Layout,
+    cell::RefCell,
+    collections::HashMap,
+    convert::TryInto,
+    ffi::OsStr,
+    io::Write,
+    mem::{align_of, size_of},
+    rc::Rc,
+};
 use tiny_keccak::{Hasher, Keccak};
 
 mod solana_tests;
@@ -62,6 +65,7 @@ struct VirtualMachine {
     logs: String,
     events: Vec<Vec<Vec<u8>>>,
     return_data: Option<(Account, Vec<u8>)>,
+    call_test: HashMap<Pubkey, fn(instr: &Instruction)>,
 }
 
 #[derive(Clone)]
@@ -115,12 +119,12 @@ fn build_solidity(src: &str) -> VirtualMachine {
 
     cache.set_file_contents("test.sol", src.to_string());
 
-    let mut ns = solang::parse_and_resolve("test.sol", &mut cache, Target::Solana);
+    let mut ns = solang::parse_and_resolve(OsStr::new("test.sol"), &mut cache, Target::Solana);
 
     // codegen all the contracts; some additional errors/warnings will be detected here
     codegen(&mut ns, &Options::default());
 
-    diagnostics::print_messages(&cache, &ns, false);
+    ns.print_diagnostics_in_plain(&cache, false);
 
     let context = inkwell::context::Context::create();
 
@@ -227,6 +231,7 @@ fn build_solidity(src: &str) -> VirtualMachine {
         logs: String::new(),
         events: Vec::new(),
         return_data: None,
+        call_test: HashMap::new(),
     }
 }
 
@@ -246,6 +251,7 @@ fn serialize_parameters(
     let mut refs = Vec::new();
     let mut v: Vec<u8> = Vec::new();
 
+    #[allow(clippy::ptr_arg)]
     fn serialize_account(
         v: &mut Vec<u8>,
         refs: &mut Vec<AccountRef>,
@@ -255,7 +261,7 @@ fn serialize_parameters(
         // dup_info
         v.write_u8(0xff).unwrap();
         // signer
-        v.write_u8(1).unwrap();
+        v.write_u8(0).unwrap();
         // is_writable
         v.write_u8(1).unwrap();
         // executable
@@ -763,7 +769,7 @@ pub struct Instruction {
     pub data: Vec<u8>,
 }
 
-#[derive(Debug, PartialEq, Clone)]
+#[derive(Debug, PartialEq, Hash, Eq, Clone)]
 pub struct Pubkey([u8; 32]);
 
 impl Pubkey {
@@ -969,7 +975,9 @@ impl<'a> SyscallObject<UserError> for SyscallInvokeSignedC<'a> {
 
             context.return_data = None;
 
-            if instruction.program_id.is_system_instruction() {
+            if let Some(handle) = context.call_test.get(&instruction.program_id) {
+                handle(&instruction);
+            } else if instruction.program_id.is_system_instruction() {
                 match bincode::deserialize::<u32>(&instruction.data).unwrap() {
                     0 => {
                         let create_account: CreateAccount =
@@ -1037,9 +1045,9 @@ impl<'a> SyscallObject<UserError> for SyscallInvokeSignedC<'a> {
                         assert_eq!(create_account.instruction, 3);
 
                         let mut hasher = Sha256::new();
-                        hasher.update(create_account.base);
+                        hasher.update(&create_account.base);
                         hasher.update(create_account.seed);
-                        hasher.update(create_account.program_id);
+                        hasher.update(&create_account.program_id);
 
                         let hash = hasher.finalize();
 
@@ -1189,7 +1197,7 @@ impl VirtualMachine {
             .register_syscall_by_name(b"sol_log_data", SyscallLogData::call)
             .unwrap();
 
-        let executable = <dyn Executable<UserError, TestInstructionMeter>>::from_elf(
+        let executable = Executable::<UserError, TestInstructionMeter>::from_elf(
             &self.account_data[&program.program].data,
             None,
             Config::default(),
@@ -1198,7 +1206,7 @@ impl VirtualMachine {
         .expect("should work");
 
         let mut vm = EbpfVm::<UserError, TestInstructionMeter>::new(
-            executable.as_ref(),
+            &executable,
             &mut heap,
             &mut parameter_bytes,
         )
@@ -1295,6 +1303,7 @@ impl VirtualMachine {
         };
 
         let res = self.execute(&calldata, &[]);
+        println!("res:{:?}", res);
         assert!(matches!(res, Ok(0)));
     }
 
@@ -1524,7 +1533,7 @@ impl VirtualMachine {
                     .map(|topic| {
                         let topic: [u8; 32] = topic.try_into().unwrap();
 
-                        ethereum_types::H256::from(topic)
+                        H256::from(topic)
                     })
                     .collect();
                 let data = fields[1].clone();
@@ -1534,38 +1543,3 @@ impl VirtualMachine {
             .collect()
     }
 }
-
-pub fn parse_and_resolve(src: &'static str, target: Target) -> ast::Namespace {
-    let mut cache = FileResolver::new();
-
-    cache.set_file_contents("test.sol", src.to_string());
-
-    solang::parse_and_resolve("test.sol", &mut cache, target)
-}
-
-pub fn first_error(errors: Vec<ast::Diagnostic>) -> String {
-    match errors.iter().find(|m| m.level == ast::Level::Error) {
-        Some(m) => m.message.to_owned(),
-        None => panic!("no errors found"),
-    }
-}
-
-pub fn no_errors(errors: Vec<ast::Diagnostic>) {
-    assert!(
-        errors
-            .iter()
-            .filter(|m| m.level == ast::Level::Error)
-            .count()
-            == 0
-    );
-}
-
-pub fn no_warnings_errors(errors: Vec<ast::Diagnostic>) {
-    assert!(
-        errors
-            .iter()
-            .filter(|m| m.level == ast::Level::Error || m.level == ast::Level::Warning)
-            .count()
-            == 0
-    );
-}
