@@ -1,7 +1,7 @@
 mod abi;
 mod accessor;
+mod account_info;
 mod arrays;
-mod assembly;
 mod balance;
 mod builtin;
 mod call;
@@ -12,6 +12,8 @@ mod events;
 mod expressions;
 mod hash;
 mod mappings;
+mod math;
+mod modifiers;
 mod primitives;
 mod rational;
 mod returns;
@@ -20,3 +22,4 @@ mod simple;
 mod storage;
 mod unused_variable_elimination;
 mod using;
+mod vector_to_slice;
