@@ -1,8 +1,5 @@
-use parity_scale_codec::Encode;
-use parity_scale_codec_derive::{Decode, Encode};
-
-use crate::{build_solidity, first_error, parse_and_resolve};
-use solang::Target;
+use crate::build_solidity;
+use parity_scale_codec::{Decode, Encode};
 
 #[derive(Debug, PartialEq, Encode, Decode)]
 struct RevertReturn(u32, String);
@@ -162,64 +159,6 @@ fn contract_already_exists() {
 
 #[test]
 fn try_catch_external_calls() {
-    let ns = parse_and_resolve(
-        r##"
-        contract c {
-            function test() public {
-                other o = new other();
-                int32 x = 0;
-                try o.test() returns (int32) {
-                    x = 1;
-                } catch (string) {
-                    x = 2;
-                }
-                assert(x == 1);
-            }
-        }
-
-        contract other {
-            function test() public returns (int32, bool) {
-                return (102, true);
-            }
-        }
-        "##,
-        Target::default_substrate(),
-    );
-
-    assert_eq!(
-        first_error(ns.diagnostics),
-        "try returns list has 1 entries while function returns 2 values"
-    );
-
-    let ns = parse_and_resolve(
-        r##"
-        contract c {
-            function test() public {
-                other o = new other();
-                int32 x = 0;
-                try o.test() returns (int32, int[2] storage) {
-                    x = 1;
-                } catch (string) {
-                    x = 2;
-                }
-                assert(x == 1);
-            }
-        }
-
-        contract other {
-            function test() public returns (int32, bool) {
-                return (102, true);
-            }
-        }
-        "##,
-        Target::default_substrate(),
-    );
-
-    assert_eq!(
-        first_error(ns.diagnostics),
-        "type ‘int256[2] storage’ does not match return value of function ‘bool’"
-    );
-
     let mut runtime = build_solidity(
         r##"
         contract c {
@@ -271,123 +210,6 @@ fn try_catch_external_calls() {
 
     runtime.function_expect_failure("test", Vec::new());
 
-    let ns = parse_and_resolve(
-        r##"
-        contract c {
-            function test() public {
-                other o = new other();
-                int32 x = 0;
-                try o.test() returns (int32, bool) {
-                    x = 1;
-                } catch (string) {
-                    x = 2;
-                }
-                assert(x == 1);
-            }
-        }
-
-        contract other {
-            function test() public returns (int32, bool) {
-                return (102, true);
-            }
-        }
-        "##,
-        Target::default_substrate(),
-    );
-
-    assert_eq!(
-        first_error(ns.diagnostics),
-        "catch can only take ‘bytes memory’, not ‘string’"
-    );
-
-    let ns = parse_and_resolve(
-        r##"
-        contract c {
-            function test() public {
-                other o = new other();
-                int32 x = 0;
-                try o.test() returns (int32 x, bool) {
-                    x = 1;
-                } catch (string) {
-                    x = 2;
-                }
-                assert(x == 1);
-            }
-        }
-
-        contract other {
-            function test() public returns (int32, bool) {
-                return (102, true);
-            }
-        }
-        "##,
-        Target::default_substrate(),
-    );
-
-    assert_eq!(first_error(ns.diagnostics), "x is already declared");
-
-    let ns = parse_and_resolve(
-        r##"
-        contract c {
-            function test() public {
-                other o = new other();
-                int32 x = 0;
-                try o.test() returns (int32 bla, bool) {
-                    x = bla;
-                } catch Foo(bytes memory f) {
-                    x = 105;
-                } catch (string) {
-                    x = 2;
-                }
-                assert(x == 1);
-            }
-        }
-
-        contract other {
-            function test() public returns (int32, bool) {
-                return (102, true);
-            }
-        }
-        "##,
-        Target::default_substrate(),
-    );
-
-    assert_eq!(
-        first_error(ns.diagnostics),
-        "only catch ‘Error’ is supported, not ‘Foo’"
-    );
-
-    let ns = parse_and_resolve(
-        r##"
-        contract c {
-            function test() public {
-                other o = new other();
-                int32 x = 0;
-                try o.test() returns (int32 bla, bool) {
-                    x = bla;
-                } catch Error(bytes memory f) {
-                    x = 105;
-                } catch (string) {
-                    x = 2;
-                }
-                assert(x == 1);
-            }
-        }
-
-        contract other {
-            function test() public returns (int32, bool) {
-                return (102, true);
-            }
-        }
-        "##,
-        Target::default_substrate(),
-    );
-
-    assert_eq!(
-        first_error(ns.diagnostics),
-        "catch Error(...) can only take ‘string memory’, not ‘bytes’"
-    );
-
     let mut runtime = build_solidity(
         r##"
         contract c {
@@ -505,60 +327,6 @@ fn try_catch_external_calls() {
 
 #[test]
 fn try_catch_constructor() {
-    let ns = parse_and_resolve(
-        r##"
-        contract c {
-            function test() public {
-                try new other() returns (int32) {
-                    x = 1;
-                } catch (string) {
-                    x = 2;
-                }
-                assert(x == 1);
-            }
-        }
-
-        contract other {
-            function test() public returns (int32, bool) {
-                return (102, true);
-            }
-        }
-        "##,
-        Target::default_substrate(),
-    );
-
-    assert_eq!(
-        first_error(ns.diagnostics),
-        "type ‘int32’ does not match return value of function ‘contract other’"
-    );
-
-    let ns = parse_and_resolve(
-        r##"
-        contract c {
-            function test() public {
-                try new other() returns (int32, int[2] storage) {
-                    x = 1;
-                } catch (string) {
-                    x = 2;
-                }
-                assert(x == 1);
-            }
-        }
-
-        contract other {
-            function test() public returns (int32, bool) {
-                return (102, true);
-            }
-        }
-        "##,
-        Target::default_substrate(),
-    );
-
-    assert_eq!(
-        first_error(ns.diagnostics),
-        "constructor returns single contract, not 2 values"
-    );
-
     let mut runtime = build_solidity(
         r##"
         contract c {
@@ -635,125 +403,6 @@ fn try_catch_constructor() {
     );
 
     runtime.function_expect_failure("test", Vec::new());
-
-    let ns = parse_and_resolve(
-        r##"
-        contract c {
-            function test() public {
-                try new int32[](2) returns (int32, bool) {
-                    x = 1;
-                } catch (string) {
-                    x = 2;
-                }
-                assert(x == 1);
-            }
-        }
-
-        contract other {
-            function test() public returns (int32, bool) {
-                return (102, true);
-            }
-        }
-        "##,
-        Target::default_substrate(),
-    );
-
-    assert_eq!(
-        first_error(ns.diagnostics),
-        "try only supports external calls or constructor calls"
-    );
-
-    let ns = parse_and_resolve(
-        r##"
-        contract c {
-            function f() public {
-                x : 1
-            }
-        }"##,
-        Target::default_substrate(),
-    );
-
-    assert_eq!(
-        first_error(ns.diagnostics),
-        "expected code block, not list of named arguments"
-    );
-
-    let ns = parse_and_resolve(
-        r##"
-        contract c {
-            function test() public {
-                try new other()
-                catch (string) {
-                    x = 2;
-                }
-                assert(x == 1);
-            }
-        }
-
-        contract other {
-            function test() public  {
-            }
-        }
-        "##,
-        Target::default_substrate(),
-    );
-
-    assert_eq!(
-        first_error(ns.diagnostics),
-        "code block missing for no catch"
-    );
-
-    let ns = parse_and_resolve(
-        r##"
-        contract c {
-            function test() public {
-                try new other() {
-                    x = 1;
-                } {
-                    x= 5;
-                }
-                catch (string) {
-                    x = 2;
-                }
-                assert(x == 1);
-            }
-        }
-
-        contract other {
-            function test() public  {
-            }
-        }
-        "##,
-        Target::default_substrate(),
-    );
-
-    assert_eq!(first_error(ns.diagnostics), "unexpected code block");
-
-    let ns = parse_and_resolve(
-        r##"
-        contract c {
-            function test(other o) public {
-                try o.test() {
-                    x = 1;
-                } {
-                    x= 5;
-                }
-                catch (string) {
-                    x = 2;
-                }
-                assert(x == 1);
-            }
-        }
-
-        contract other {
-            function test() public  {
-            }
-        }
-        "##,
-        Target::default_substrate(),
-    );
-
-    assert_eq!(first_error(ns.diagnostics), "unexpected code block");
 }
 
 #[test]
@@ -957,70 +606,6 @@ fn payable_functions() {
     runtime.function("get_x", Vec::new());
 
     assert_eq!(runtime.vm.output, Ret(2).encode());
-
-    let ns = parse_and_resolve(
-        r##"
-        contract c {
-            receive() public {
-
-            }
-        }
-        "##,
-        Target::default_substrate(),
-    );
-
-    assert_eq!(
-        first_error(ns.diagnostics),
-        "receive function must be declared external"
-    );
-
-    let ns = parse_and_resolve(
-        r##"
-        contract c {
-            receive() external  {
-
-            }
-        }
-        "##,
-        Target::default_substrate(),
-    );
-
-    assert_eq!(
-        first_error(ns.diagnostics),
-        "receive function must be declared payable"
-    );
-
-    let ns = parse_and_resolve(
-        r##"
-        contract c {
-            fallback() payable external {
-
-            }
-        }
-        "##,
-        Target::default_substrate(),
-    );
-
-    assert_eq!(
-        first_error(ns.diagnostics),
-        "fallback function must not be declare payable, use ‘receive() external payable’ instead"
-    );
-
-    let ns = parse_and_resolve(
-        r##"
-        contract c {
-            fallback() public {
-
-            }
-        }
-        "##,
-        Target::default_substrate(),
-    );
-
-    assert_eq!(
-        first_error(ns.diagnostics),
-        "fallback function must be declared external"
-    );
 }
 
 #[test]
