@@ -19,8 +19,8 @@ mod loops;
 mod mappings;
 mod modifier;
 mod primitives;
+mod storage;
 mod strings;
 mod structs;
-mod tags;
 mod value;
 mod variables;
