@@ -1,20 +1,22 @@
-use solang::codegen::{codegen, Options};
+use solang::codegen::{codegen, OptimizationLevel, Options};
 use solang::file_resolver::FileResolver;
 use solang::sema::ast::Diagnostic;
-use solang::sema::ast::{Level, Namespace};
+use solang::sema::ast::Namespace;
 use solang::{parse_and_resolve, Target};
+use std::ffi::OsStr;
 
 fn parse_and_codegen(src: &'static str) -> Namespace {
     let mut cache = FileResolver::new();
     cache.set_file_contents("test.sol", src.to_string());
-    let mut ns = parse_and_resolve("test.sol", &mut cache, Target::Solana);
+    let mut ns = parse_and_resolve(OsStr::new("test.sol"), &mut cache, Target::Ewasm);
 
     let opt = Options {
         dead_storage: false,
         constant_folding: false,
         strength_reduce: false,
         vector_to_slice: false,
-        opt_level: inkwell::OptimizationLevel::Default,
+        common_subexpression_elimination: false,
+        opt_level: OptimizationLevel::Default,
         math_overflow_check: false,
     };
 
@@ -23,16 +25,6 @@ fn parse_and_codegen(src: &'static str) -> Namespace {
     ns
 }
 
-fn get_errors(ns: &Namespace) -> Vec<&Diagnostic> {
-    let mut vec = Vec::new();
-    for diag in &ns.diagnostics {
-        if matches!(diag.level, Level::Error) {
-            vec.push(diag);
-        }
-    }
-    vec
-}
-
 fn contains_error_message_and_notes(
     errors: &[&Diagnostic],
     message: &str,
@@ -76,7 +68,7 @@ fn used_before_being_defined() {
     "#;
 
     let ns = parse_and_codegen(file);
-    let errors = get_errors(&ns);
+    let errors = ns.diagnostics.errors();
     assert_eq!(errors.len(), 1);
     assert_eq!(errors[0].message, "Variable 'b32' is undefined");
     assert_eq!(errors[0].notes.len(), 2);
@@ -124,7 +116,7 @@ fn struct_as_ref() {
     "#;
 
     let ns = parse_and_codegen(file);
-    let errors = get_errors(&ns);
+    let errors = ns.diagnostics.errors();
     assert_eq!(errors.len(), 1);
     assert_eq!(errors[0].message, "Variable 'f' is undefined");
     assert_eq!(errors[0].notes.len(), 3);
@@ -173,7 +165,7 @@ fn struct_as_ref() {
     "#;
 
     let ns = parse_and_codegen(file);
-    let errors = get_errors(&ns);
+    let errors = ns.diagnostics.errors();
     assert_eq!(errors.len(), 0);
 }
 
@@ -193,7 +185,7 @@ fn while_loop() {
     }
     "#;
     let ns = parse_and_codegen(file);
-    let errors = get_errors(&ns);
+    let errors = ns.diagnostics.errors();
     assert_eq!(errors.len(), 1);
     assert_eq!(errors[0].message, "Variable 's' is undefined");
     assert_eq!(errors[0].notes.len(), 1);
@@ -221,7 +213,7 @@ fn while_loop() {
     "#;
 
     let ns = parse_and_codegen(file);
-    let errors = get_errors(&ns);
+    let errors = ns.diagnostics.errors();
     assert_eq!(errors.len(), 1);
     assert_eq!(errors[0].message, "Variable 's' is undefined");
     assert_eq!(errors[0].notes.len(), 1);
@@ -251,7 +243,7 @@ fn while_loop() {
     "#;
 
     let ns = parse_and_codegen(file);
-    let errors = get_errors(&ns);
+    let errors = ns.diagnostics.errors();
     assert_eq!(errors.len(), 0);
 }
 
@@ -275,7 +267,7 @@ fn for_loop() {
     "#;
 
     let ns = parse_and_codegen(file);
-    let errors = get_errors(&ns);
+    let errors = ns.diagnostics.errors();
     assert_eq!(errors.len(), 2);
     assert!(contains_error_message_and_notes(
         &errors,
@@ -310,7 +302,7 @@ fn for_loop() {
     }
     "#;
     let ns = parse_and_codegen(file);
-    let errors = get_errors(&ns);
+    let errors = ns.diagnostics.errors();
     assert_eq!(errors.len(), 0);
 }
 
@@ -335,7 +327,7 @@ fn do_while_loop() {
     "#;
 
     let ns = parse_and_codegen(file);
-    let errors = get_errors(&ns);
+    let errors = ns.diagnostics.errors();
     assert_eq!(errors.len(), 0);
 }
 
@@ -358,7 +350,7 @@ fn if_else_condition() {
     "#;
 
     let ns = parse_and_codegen(file);
-    let errors = get_errors(&ns);
+    let errors = ns.diagnostics.errors();
     assert_eq!(errors.len(), 1);
     assert_eq!(errors[0].message, "Variable 'o' is undefined");
     assert_eq!(errors[0].notes.len(), 1);
@@ -388,7 +380,7 @@ fn if_else_condition() {
     "#;
 
     let ns = parse_and_codegen(file);
-    let errors = get_errors(&ns);
+    let errors = ns.diagnostics.errors();
     assert_eq!(errors.len(), 1);
     assert_eq!(errors[0].message, "Variable 'o' is undefined");
     assert_eq!(errors[0].notes.len(), 1);
@@ -416,7 +408,7 @@ fn if_else_condition() {
     "#;
 
     let ns = parse_and_codegen(file);
-    let errors = get_errors(&ns);
+    let errors = ns.diagnostics.errors();
     assert_eq!(errors.len(), 0);
 }
 
@@ -433,7 +425,7 @@ fn array() {
     }
     "#;
     let ns = parse_and_codegen(file);
-    let errors = get_errors(&ns);
+    let errors = ns.diagnostics.errors();
     assert_eq!(errors.len(), 1);
     assert_eq!(errors[0].message, "Variable 'vec' is undefined");
     assert_eq!(errors[0].notes.len(), 1);
@@ -458,7 +450,7 @@ fn array() {
     "#;
 
     let ns = parse_and_codegen(file);
-    let errors = get_errors(&ns);
+    let errors = ns.diagnostics.errors();
     assert_eq!(errors.len(), 0);
 }
 
@@ -491,7 +483,7 @@ contract test {
     "#;
 
     let ns = parse_and_codegen(file);
-    let errors = get_errors(&ns);
+    let errors = ns.diagnostics.errors();
     assert!(contains_error_message_and_notes(
         &errors,
         "Variable 'o' is undefined",
@@ -531,7 +523,7 @@ fn basic_types() {
 }
     "#;
     let ns = parse_and_codegen(file);
-    let errors = get_errors(&ns);
+    let errors = ns.diagnostics.errors();
     assert!(contains_error_message_and_notes(
         &errors,
         "Variable 'bt' is undefined",
@@ -592,7 +584,7 @@ contract test {
 }
     "#;
     let ns = parse_and_codegen(file);
-    let errors = get_errors(&ns);
+    let errors = ns.diagnostics.errors();
     assert!(contains_error_message_and_notes(
         &errors,
         "Variable 'b' is undefined",
@@ -640,7 +632,7 @@ fn try_catch() {
     // "#;
     //
     // let ns = parse_and_codegen(file);
-    // let errors = get_errors(&ns);
+    // let errors = ns.diagnostics.errors();
     // assert_eq!(errors.len(), 1);
     // assert_eq!(errors[0].message, "Variable 'r' is undefined");
     // assert_eq!(errors[0].notes.len(), 1);
@@ -673,7 +665,7 @@ fn try_catch() {
     "#;
 
     let ns = parse_and_codegen(file);
-    let errors = get_errors(&ns);
+    let errors = ns.diagnostics.errors();
     assert_eq!(errors.len(), 0);
 
     let file = r#"
@@ -701,7 +693,7 @@ fn try_catch() {
     "#;
 
     let ns = parse_and_codegen(file);
-    let errors = get_errors(&ns);
+    let errors = ns.diagnostics.errors();
     assert_eq!(errors.len(), 1);
     assert_eq!(errors[0].message, "Variable 'r' is undefined");
     assert_eq!(errors[0].notes.len(), 1);
@@ -735,7 +727,7 @@ fn try_catch() {
     "#;
 
     let ns = parse_and_codegen(file);
-    let errors = get_errors(&ns);
+    let errors = ns.diagnostics.errors();
     assert_eq!(errors.len(), 1);
     assert_eq!(errors[0].message, "Variable 'r' is undefined");
     assert_eq!(errors[0].notes.len(), 1);
