@@ -9,13 +9,11 @@ export let platformEol: string;
 export async function activate(docUri: vscode.Uri) {
   // The extensionId is `publisher.name` from package.json
 
-  const ext: vscode.Extension<any> | undefined = vscode.extensions.getExtension('solang.solang');
-
-  let extn_act;
+  const ext = vscode.extensions.getExtension('solang.solang');
 
   if (ext) {
     try {
-      extn_act = await ext.activate();
+      await ext.activate();
     } catch (e) {
       console.error(e);
     }
